@@ -24,7 +24,6 @@
 #include "ephy-embed-container.h"
 #include "ephy-embed-shell.h"
 #include "ephy-lib-type-builtins.h"
-#include "ephy-output-encoding.h"
 #include "ephy-settings.h"
 #include "ephy-web-view.h"
 
@@ -123,7 +122,8 @@ enum_nick (GType enum_type,
   return nick;
 }
 
-static char *
+static
+char *
 readability_get_property_string (WebKitJavascriptResult *js_result,
                                  char                   *property)
 {
@@ -157,9 +157,7 @@ readability_js_finish_cb (GObject      *object,
   g_autoptr (WebKitJavascriptResult) js_result = NULL;
   g_autoptr (GError) error = NULL;
   g_autofree gchar *byline = NULL;
-  g_autofree gchar *encoded_byline = NULL;
   g_autofree gchar *content = NULL;
-  g_autofree gchar *encoded_title = NULL;
   g_autoptr (GString) html = NULL;
   g_autoptr (GBytes) style_css = NULL;
   const gchar *title;
@@ -176,14 +174,10 @@ readability_js_finish_cb (GObject      *object,
 
   byline = readability_get_property_string (js_result, "byline");
   content = readability_get_property_string (js_result, "content");
-  title = webkit_web_view_get_title (web_view);
 
-  encoded_byline = byline ? ephy_encode_for_html_entity (byline) : g_strdup ("");
-  encoded_title = ephy_encode_for_html_entity (title);
-
-  html = g_string_new (NULL);
+  html = g_string_new ("");
   style_css = g_resources_lookup_data ("/org/gnome/epiphany/readability/reader.css", G_RESOURCE_LOOKUP_FLAGS_NONE, NULL);
-
+  title = webkit_web_view_get_title (web_view);
   font_style = enum_nick (EPHY_TYPE_PREFS_READER_FONT_STYLE,
                           g_settings_get_enum (EPHY_SETTINGS_READER,
                                                EPHY_PREFS_READER_FONT_STYLE));
@@ -193,8 +187,7 @@ readability_js_finish_cb (GObject      *object,
 
   g_string_append_printf (html, "<style>%s</style>"
                           "<title>%s</title>"
-                          "<meta http-equiv='Content-Type' content='text/html;' charset='UTF-8'>" \
-                          "<meta http-equiv='Content-Security-Policy' content=\"script-src 'none'\">" \
+                          "<meta http-equiv=\"Content-Type\" content=\"text/html;\" charset=\"UTF-8\">" \
                           "<body class='%s %s'>"
                           "<article>"
                           "<h2>"
@@ -205,27 +198,13 @@ readability_js_finish_cb (GObject      *object,
                           "</i>"
                           "<hr>",
                           (gchar *)g_bytes_get_data (style_css, NULL),
-                          encoded_title,
+                          title,
                           font_style,
                           color_scheme,
-                          encoded_title,
-                          encoded_byline);
-
-  /* We cannot encode the page content because it contains HTML tags inserted by
-   * Readability.js. Upstream recommends that we use an XSS sanitizer like
-   * DOMPurify plus Content-Security-Policy, but I'm not keen on adding more
-   * bundled JS dependencies, and we have an advantage over Firefox in that we
-   * don't need scripts to work at this point. So instead the above CSP
-   * completely blocks all scripts, which should hopefully obviate the need for
-   * a DOM purifier.
-   *
-   * Note the encoding for page title and byline is still required, as they're
-   * not supposed to contain markup, and Readability.js unescapes them before
-   * returning them to us.
-   */
+                          title,
+                          byline != NULL ? byline : "");
   g_string_append (html, content);
   g_string_append (html, "</article>");
-  g_string_append (html, "</body>");
 
   finish_uri_scheme_request (request, g_strdup (html->str), NULL);
 }
@@ -275,14 +254,14 @@ ephy_reader_request_begin_get_source_from_uri (EphyReaderRequest *request,
 static void
 ephy_reader_request_start (EphyReaderRequest *request)
 {
-  g_autoptr (SoupURI) soup_uri = NULL;
+  g_autoptr (GUri) uri = NULL;
   const char *original_uri;
   WebKitWebView *web_view;
 
   original_uri = webkit_uri_scheme_request_get_uri (request->scheme_request);
-  soup_uri = soup_uri_new (original_uri);
+  uri = g_uri_parse (original_uri, G_URI_FLAGS_NONE, NULL);
 
-  if (!soup_uri) {
+  if (!uri) {
     /* Can't assert because user could theoretically input something weird */
     GError *error = g_error_new (WEBKIT_NETWORK_ERROR,
                                  WEBKIT_NETWORK_ERROR_FAILED,
@@ -304,15 +283,11 @@ ephy_reader_request_start (EphyReaderRequest *request)
   if (web_view) {
     ephy_reader_request_begin_get_source_from_web_view (request, web_view);
   } else {
-    const char *source_uri;
-
     /* Extract URI:
      * ephy-reader:https://example.com/whatever?xyz into https://example.com/whatever?xyz
      */
-    source_uri = soup_uri_to_string (soup_uri, TRUE);
-    g_assert (source_uri);
-
-    ephy_reader_request_begin_get_source_from_uri (request, source_uri);
+    g_assert (g_str_has_prefix (original_uri, "ephy-reader:"));
+    ephy_reader_request_begin_get_source_from_uri (request, original_uri + strlen ("ephy-reader:"));
   }
 
   request->source_handler->outstanding_requests =
