@@ -23,7 +23,6 @@
 
 #include "ephy-embed-container.h"
 #include "ephy-embed-shell.h"
-#include "ephy-output-encoding.h"
 #include "ephy-web-view.h"
 
 #include <gio/gio.h>
@@ -110,9 +109,7 @@ web_resource_data_cb (WebKitWebResource     *resource,
                       EphyViewSourceRequest *request)
 {
   g_autofree guchar *data = NULL;
-  g_autofree char *data_str = NULL;
-  g_autofree char *encoded_str = NULL;
-  g_autofree char *encoded_uri = NULL;
+  g_autofree char *escaped_str = NULL;
   g_autoptr (GError) error = NULL;
   g_autofree char *html = NULL;
   gsize length;
@@ -123,16 +120,11 @@ web_resource_data_cb (WebKitWebResource     *resource,
     return;
   }
 
-  /* Convert data to a string */
-  data_str = g_malloc (length + 1);
-  memcpy (data_str, data, length);
-  data_str[length] = '\0';
-
-  encoded_str = ephy_encode_for_html_entity (data_str);
-  encoded_uri = ephy_encode_for_html_entity (webkit_web_resource_get_uri (resource));
+  /* Warning: data is not a string, so we pass length here because it's not NUL-terminated. */
+  escaped_str = g_markup_escape_text ((const char *)data, length);
 
   html = g_strdup_printf ("<head>"
-                          "  <link rel='stylesheet' href='ephy-resource:///org/gnome/epiphany/highlightjs/nnfx.css' media='(prefers-color-scheme: no-preference), (prefers-color-scheme: light)'>"
+                          "  <link rel='stylesheet' href='ephy-resource:///org/gnome/epiphany/highlightjs/nnfx-light.css' media='(prefers-color-scheme: no-preference), (prefers-color-scheme: light)'>"
                           "  <link rel='stylesheet' href='ephy-resource:///org/gnome/epiphany/highlightjs/nnfx-dark.css' media='(prefers-color-scheme: dark)'>"
                           "  <link rel='stylesheet' href='ephy-resource:///org/gnome/epiphany/highlightjs/epiphany.css'>"
                           "  <title>%s</title>"
@@ -140,12 +132,12 @@ web_resource_data_cb (WebKitWebResource     *resource,
                           "<body class='hljs'>"
                           "  <script src='ephy-resource:///org/gnome/epiphany/highlightjs/highlight.js'></script>"
                           "  <script src='ephy-resource:///org/gnome/epiphany/highlightjs/highlightjs-line-numbers.js'></script>"
-                          "  <script>hljs.initHighlightingOnLoad();"
+                          "  <script>hljs.highlightAll();"
                           "          hljs.initLineNumbersOnLoad();</script>"
                           "  <pre><code class='html'>%s</code></pre>"
                           "</body>",
-                          encoded_uri,
-                          encoded_str);
+                          webkit_web_resource_get_uri (resource),
+                          escaped_str);
 
   finish_uri_scheme_request (request, g_steal_pointer (&html), NULL);
 }
@@ -194,11 +186,12 @@ ephy_view_source_request_begin_get_source_from_uri (EphyViewSourceRequest *reque
 
 static gint
 embed_is_displaying_matching_uri (EphyEmbed *embed,
-                                  SoupURI   *uri)
+                                  GUri      *uri)
 {
   EphyWebView *web_view;
-  SoupURI *view_uri;
-  gint ret = -1;
+  g_autoptr (GUri) view_uri = NULL;
+  g_autofree char *modified_uri = NULL;
+  g_autofree char *uri_string = NULL;
 
   if (ephy_embed_has_load_pending (embed))
     return -1;
@@ -207,20 +200,18 @@ embed_is_displaying_matching_uri (EphyEmbed *embed,
   if (ephy_web_view_is_loading (web_view))
     return -1;
 
-  view_uri = soup_uri_new (ephy_web_view_get_address (web_view));
+  view_uri = g_uri_parse (ephy_web_view_get_address (web_view),
+                          G_URI_FLAGS_NONE, NULL);
   if (!view_uri)
     return -1;
 
-  soup_uri_set_fragment (view_uri, NULL);
-  ret = soup_uri_equal (view_uri, uri) ? 0 : -1;
-
-  soup_uri_free (view_uri);
-
-  return ret;
+  modified_uri = g_uri_to_string_partial (view_uri, G_URI_HIDE_FRAGMENT);
+  uri_string = g_uri_to_string (uri);
+  return strcmp (modified_uri, uri_string);
 }
 
 static WebKitWebView *
-get_web_view_matching_uri (SoupURI *uri)
+get_web_view_matching_uri (GUri *uri)
 {
   EphyEmbedShell *shell;
   GtkWindow *window;
@@ -249,10 +240,8 @@ out:
 static void
 ephy_view_source_request_start (EphyViewSourceRequest *request)
 {
-  guint port;
-  SoupURI *soup_uri;
-  char *modified_uri;
-  char *decoded_fragment;
+  g_autoptr (GUri) uri = NULL;
+  g_autoptr (GUri) converted_uri = NULL;
   const char *original_uri;
   WebKitWebView *web_view;
 
@@ -260,9 +249,9 @@ ephy_view_source_request_start (EphyViewSourceRequest *request)
     g_list_prepend (request->source_handler->outstanding_requests, request);
 
   original_uri = webkit_uri_scheme_request_get_uri (request->scheme_request);
-  soup_uri = soup_uri_new (original_uri);
+  uri = g_uri_parse (original_uri, G_URI_FLAGS_ENCODED | G_URI_FLAGS_SCHEME_NORMALIZE, NULL);
 
-  if (!soup_uri || !soup_uri->fragment) {
+  if (!uri || !g_uri_get_fragment (uri)) {
     /* Can't assert because user could theoretically input something weird */
     GError *error = g_error_new (WEBKIT_NETWORK_ERROR,
                                  WEBKIT_NETWORK_ERROR_FAILED,
@@ -273,26 +262,26 @@ ephy_view_source_request_start (EphyViewSourceRequest *request)
     return;
   }
 
-  /* Convert e.g. ephy-source://gnome.org#https to https://gnome.org, taking
-   * care to prevent soup_uri_set_scheme() from forcing the default port.
-   */
-  decoded_fragment = soup_uri_decode (soup_uri->fragment);
-  port = soup_uri_get_port (soup_uri);
-  soup_uri_set_scheme (soup_uri, decoded_fragment);
-  soup_uri_set_port (soup_uri, port);
-  soup_uri_set_fragment (soup_uri, NULL);
-  modified_uri = soup_uri_to_string (soup_uri, FALSE);
-  g_assert (modified_uri);
+  /* Convert e.g. ephy-source://gnome.org#https to https://gnome.org */
+  converted_uri = g_uri_build (g_uri_get_flags (uri),
+                               g_uri_get_fragment (uri),
+                               g_uri_get_userinfo (uri),
+                               g_uri_get_host (uri),
+                               g_uri_get_port (uri),
+                               g_uri_get_path (uri),
+                               g_uri_get_query (uri),
+                               NULL);
+  g_assert (converted_uri);
 
-  web_view = get_web_view_matching_uri (soup_uri);
+  web_view = get_web_view_matching_uri (converted_uri);
   if (web_view)
     ephy_view_source_request_begin_get_source_from_web_view (request, WEBKIT_WEB_VIEW (web_view));
-  else
-    ephy_view_source_request_begin_get_source_from_uri (request, modified_uri);
+  else {
+    g_autofree char *modified_uri = NULL;
 
-  g_free (decoded_fragment);
-  g_free (modified_uri);
-  soup_uri_free (soup_uri);
+    modified_uri = g_uri_to_string (converted_uri);
+    ephy_view_source_request_begin_get_source_from_uri (request, modified_uri);
+  }
 }
 
 static void
