@@ -1,20 +1,22 @@
 'use strict';
 
+// eslint-disable-next-line no-var
 var Ephy = {};
 
 Ephy.getAppleMobileWebAppCapable = function()
 {
-    let metas = document.getElementsByTagName('meta');
+    const metas = document.getElementsByTagName('meta');
 
     for (let i = 0; i < metas.length; i++) {
-        let meta = metas[i];
+        const meta = metas[i];
 
-        if (meta.name == 'apple-mobile-web-app-capable' && meta.getAttribute('content') == 'yes')
+        // https://developer.apple.com/library/archive/documentation/AppleApplications/Reference/SafariHTMLRef/Articles/MetaTags.html
+        if (meta.name === 'apple-mobile-web-app-capable' && meta.getAttribute('content') === 'yes')
             return true;
     }
 
     return false;
-}
+};
 
 Ephy.getWebAppTitle = function()
 {
@@ -22,37 +24,41 @@ Ephy.getWebAppTitle = function()
 
     for (let i = 0; i < metas.length; i++) {
         const meta = metas[i];
-        if (meta.name == 'application-name')
+        // https://developer.mozilla.org/en-US/docs/Web/HTML/Element/meta/name#standard_metadata_names_defined_in_the_html_specification
+        if (meta.name === 'application-name')
             return meta.content;
 
+        // https://developer.apple.com/library/archive/documentation/AppleApplications/Reference/SafariWebContent/ConfiguringWebApplications/ConfiguringWebApplications.html
         if (meta.name === 'apple-mobile-web-app-title')
             return meta.content;
 
+        // https://ogp.me/
         // og:site_name is read from the property attribute (standard), but is
         // commonly seen on the web in the name attribute. Both are supported.
-        if (meta.getAttribute('property') == 'og:site_name' || meta.name == 'og:site_name')
+        if (meta.getAttribute('property') === 'og:site_name' || meta.name === 'og:site_name')
             return meta.content;
     }
 
-    // Fallback to document title
+    // document title
+    // Prefer HTML tag over dynamic updates
     const titles = document.head.getElementsByTagName('title');
-
     if (titles.length > 0) {
       const title = titles[titles.length - 1];
 
       if (title && title.innerText)
           return title.innerText;
     }
+    if (document.title)
+        return document.title;
 
-    // Last resort: hostname
-    return document.location.hostname;
-}
+    // set_default_application_title will fallback to the formatted hostname
+    return null;
+};
 
 Ephy.getWebAppIcon = function(baseURL)
 {
     let htmlIconURL = null;
     let msIconURL = null;
-    let appleTouchIconURL = null;
     let largestIconSize = 0;
     let iconColor = null;
     let ogpIcon = null;
@@ -62,16 +68,16 @@ Ephy.getWebAppIcon = function(baseURL)
     for (let i = 0; i < links.length; i++) {
         const link = links[i];
 
-        if (link.rel == 'icon' || link.rel == 'shortcut icon' || link.rel == 'icon shortcut' || link.rel == 'shortcut-icon' || link.rel == 'apple-touch-icon' || link.rel == 'apple-touch-icon-precomposed') {
+        if (link.rel === 'icon' || link.rel === 'shortcut icon' || link.rel === 'icon shortcut' || link.rel === 'shortcut-icon' || link.rel === 'apple-touch-icon' || link.rel === 'apple-touch-icon-precomposed') {
             const sizes = link.getAttribute('sizes');
 
             if (!sizes) {
-                if (largestIconSize == 0 && (!htmlIconURL || link.rel == 'apple-touch-icon' || link.rel == 'apple-touch-icon-precomposed'))
+                if (largestIconSize === 0 && (!htmlIconURL || link.rel === 'apple-touch-icon' || link.rel === 'apple-touch-icon-precomposed'))
                   htmlIconURL = link.href;
                 continue;
             }
 
-            if (sizes == 'any') {
+            if (sizes === 'any') {
                 // "any" means a vector, and thus it will always be the largest icon.
                 htmlIconURL = link.href;
                 break;
@@ -82,7 +88,7 @@ Ephy.getWebAppIcon = function(baseURL)
                 const size = sizesList[j].toLowerCase().split('x');
 
                 // Only accept square icons.
-                if (size.length != 2 || size[0] != size[1])
+                if (size.length !== 2 || size[0] !== size[1])
                     continue;
 
                 // Only accept icons of 96 px (smallest GNOME HIG app icon) or larger.
@@ -96,17 +102,17 @@ Ephy.getWebAppIcon = function(baseURL)
         }
     }
 
-    if (largestIconSize != 0 && htmlIconURL)
+    if (largestIconSize !== 0 && htmlIconURL)
         return { 'url' : new URL(htmlIconURL, baseURL).href, 'color' : null };
 
     for (let i = 0; i < metas.length; i++) {
         const meta = metas[i];
 
-        if (meta.name == 'msapplication-TileImage')
+        if (meta.name === 'msapplication-TileImage')
             msIconURL = meta.content;
-        else if (meta.name == 'msapplication-TileColor')
+        else if (meta.name === 'msapplication-TileColor')
             iconColor = meta.content;
-        else if (meta.getAttribute('property') == 'og:image' || meta.getAttribute('itemprop') == 'image')
+        else if (meta.getAttribute('property') === 'og:image' || meta.getAttribute('itemprop') === 'image')
             ogpIcon = meta.content;
     }
 
@@ -124,7 +130,7 @@ Ephy.getWebAppIcon = function(baseURL)
 
     // Last ditch effort: just fallback to the default favicon location.
     return { 'url' : new URL('/favicon.ico', baseURL).href, 'color' : null };
-}
+};
 
 Ephy.PreFillUserMenu = class PreFillUserMenu
 {
@@ -167,15 +173,15 @@ Ephy.PreFillUserMenu = class PreFillUserMenu
 
     _onKeyDown(event)
     {
-        if (event.key == 'Escape') {
+        if (event.key === 'Escape') {
             this._removeMenu();
             return;
         }
 
-        if (event.key != 'ArrowDown' && event.key != 'ArrowUp')
+        if (event.key !== 'ArrowDown' && event.key !== 'ArrowUp')
             return;
 
-        let container = document.getElementById('ephy-user-choices-container');
+        const container = document.getElementById('ephy-user-choices-container');
         if (!container) {
             this._showMenu(!this._wasEdited);
             return;
@@ -183,10 +189,10 @@ Ephy.PreFillUserMenu = class PreFillUserMenu
 
         let newSelect = null;
         if (this._selected)
-            newSelect = event.key != 'ArrowUp' ? this._selected.previousSibling : this._selected.nextSibling;
+            newSelect = event.key !== 'ArrowUp' ? this._selected.previousSibling : this._selected.nextSibling;
 
         if (!newSelect)
-            newSelect = event.key != 'ArrowUp' ? container.firstElementChild.lastElementChild : container.firstElementChild.firstElementChild;
+            newSelect = event.key !== 'ArrowUp' ? container.firstElementChild.lastElementChild : container.firstElementChild.firstElementChild;
 
         if (newSelect) {
             this._selected = newSelect;
@@ -201,10 +207,10 @@ Ephy.PreFillUserMenu = class PreFillUserMenu
 
     _showMenu(showAll)
     {
-        let mainDiv = document.createElement('div');
+        const mainDiv = document.createElement('div');
         mainDiv.id = 'ephy-user-choices-container';
 
-        let elementRect = this._userElement.getBoundingClientRect();
+        const elementRect = this._userElement.getBoundingClientRect();
 
         // 2147483647 is the maximum value browsers will take for z-index.
         // See http://stackoverflow.com/questions/8565821/css-max-z-index-value
@@ -221,18 +227,18 @@ Ephy.PreFillUserMenu = class PreFillUserMenu
         mainDiv.style.left = elementRect.left + document.body.scrollLeft + 'px';
         mainDiv.style.top = elementRect.top + elementRect.height + document.body.scrollTop + 'px';
 
-        let ul = document.createElement('ul');
+        const ul = document.createElement('ul');
         ul.style.cssText = 'margin: 0; padding: 0;';
         ul.tabindex = -1;
         mainDiv.appendChild(ul);
 
         this._selected = null;
         for (let i = 0; i < this._users.length; i++) {
-            let user = this._users[i];
+            const user = this._users[i];
             if (!showAll && !user.startsWith(this._userElement.value))
                 continue;
 
-            let li = document.createElement('li');
+            const li = document.createElement('li');
             li.style.cssText = 'list-style-type: none ! important;' +
                 'background-image: none ! important;' +
                 'padding: 3px 6px ! important;' +
@@ -242,10 +248,10 @@ Ephy.PreFillUserMenu = class PreFillUserMenu
             li.tabindex = -1;
             ul.appendChild(li);
 
-            if (user == this._userElement.value)
+            if (user === this._userElement.value)
                 this._selected = li;
 
-            let anchor = document.createElement('a');
+            const anchor = document.createElement('a');
             anchor.style.cssText = 'font-weight: normal ! important;' +
                 'font-family: sans ! important;' +
                 'text-decoration: none ! important;' +
@@ -271,7 +277,7 @@ Ephy.PreFillUserMenu = class PreFillUserMenu
 
     _removeMenu()
     {
-        let menu = document.getElementById('ephy-user-choices-container');
+        const menu = document.getElementById('ephy-user-choices-container');
         if (menu)
             menu.parentNode.removeChild(menu);
     }
@@ -282,7 +288,7 @@ Ephy.PreFillUserMenu = class PreFillUserMenu
         this._passwordElement.value = '';
         this._manager.preFill(this._formAuth);
     }
-}
+};
 
 Ephy.formControlsAssociated = function(pageID, frameID, forms, serializer)
 {
@@ -291,20 +297,20 @@ Ephy.formControlsAssociated = function(pageID, frameID, forms, serializer)
     for (let i = 0; i < forms.length; i++) {
         if (!(forms[i] instanceof HTMLFormElement))
             continue;
-        let formManager = new Ephy.FormManager(pageID, frameID, forms[i]);
+        const formManager = new Ephy.FormManager(pageID, frameID, forms[i]);
         formManager.handlePasswordForms(serializer);
         formManager.preFillForms();
         Ephy.formManagers.push(formManager);
     }
-}
+};
 
 Ephy.handleFormSubmission = function(pageID, frameID, form)
 {
     // FIXME: Find out: is it really possible to have multiple frames with same window object???
     let formManager = null;
     for (let i = 0; i < Ephy.formManagers.length; i++) {
-        let manager = Ephy.formManagers[i];
-        if (manager.frameID() == frameID && manager.form() == form) {
+        const manager = Ephy.formManagers[i];
+        if (manager.frameID() === frameID && manager.form() === form) {
             formManager = manager;
             break;
         }
@@ -316,15 +322,15 @@ Ephy.handleFormSubmission = function(pageID, frameID, form)
     }
 
     formManager.handleFormSubmission();
-}
+};
 
 Ephy.hasModifiedForms = function()
 {
     for (let i = 0; i < document.forms.length; i++) {
-        let form = document.forms[i];
+        const form = document.forms[i];
         let modifiedInputElement = false;
         for (let j = 0; j < form.elements.length; j++) {
-            let element = form.elements[j];
+            const element = form.elements[j];
             if (!Ephy.isEdited(element))
                 continue;
 
@@ -344,7 +350,7 @@ Ephy.hasModifiedForms = function()
             }
         }
     }
-}
+};
 
 Ephy.PasswordManager = class PasswordManager
 {
@@ -358,7 +364,7 @@ Ephy.PasswordManager = class PasswordManager
 
     _takePendingPromise(id)
     {
-        let element = this._pendingPromises.find(element => element.promiseID === id);
+        const element = this._pendingPromises.find(element => element.promiseID === id);
         if (element)
             this._pendingPromises = this._pendingPromises.filter(element => element.promiseID !== id);
         return element;
@@ -368,7 +374,7 @@ Ephy.PasswordManager = class PasswordManager
     {
         Ephy.log(`Received password query response for username=${username}`);
 
-        let element = this._takePendingPromise(id);
+        const element = this._takePendingPromise(id);
         if (element) {
             if (password)
                 element.resolver({username, password});
@@ -382,7 +388,7 @@ Ephy.PasswordManager = class PasswordManager
         Ephy.log(`Querying passwords for origin=${origin}, targetOrigin=${targetOrigin}, username=${username}, usernameField=${usernameField}, passwordField=${passwordField}`);
 
         return new Promise((resolver, reject) => {
-            let promiseID = this._promiseCounter++;
+            const promiseID = this._promiseCounter++;
             Ephy.queryPassword(origin, targetOrigin, username, usernameField, passwordField, promiseID, this._pageID, this._frameID);
             this._pendingPromises.push({promiseID, resolver});
         });
@@ -413,7 +419,7 @@ Ephy.PasswordManager = class PasswordManager
     {
         Ephy.log(`Received query usernames response with users=${users}`);
 
-        let element = this._takePendingPromise(id);
+        const element = this._takePendingPromise(id);
         if (element)
             element.resolver(users);
     }
@@ -423,12 +429,12 @@ Ephy.PasswordManager = class PasswordManager
         Ephy.log(`Requesting usernames for origin=${origin}`);
 
         return new Promise((resolver, reject) => {
-            let promiseID = this._promiseCounter++;
+            const promiseID = this._promiseCounter++;
             Ephy.queryUsernames(origin, promiseID, this._pageID, this._frameID);
             this._pendingPromises.push({promiseID, resolver});
         });
     }
-}
+};
 
 Ephy.FormManager = class FormManager
 {
@@ -474,7 +480,7 @@ Ephy.FormManager = class FormManager
         if (!Ephy.shouldRememberPasswords())
             return;
 
-        let formAuth = this._generateFormAuth(true);
+        const formAuth = this._generateFormAuth(true);
         if (!formAuth) {
             Ephy.log('No pre-fillable/hookable form found');
             return;
@@ -533,17 +539,17 @@ Ephy.FormManager = class FormManager
         if (!Ephy.shouldRememberPasswords())
             return;
 
-        let formAuth = this._generateFormAuth(false);
+        const formAuth = this._generateFormAuth(false);
         if (!formAuth || !formAuth.password)
             return;
 
         let permission = Ephy.permissionsManager.permission(Ephy.PermissionType.SAVE_PASSWORD, formAuth.origin);
-        if (permission == Ephy.Permission.DENY) {
+        if (permission === Ephy.Permission.DENY) {
             Ephy.log('User/password storage permission previously denied. Not asking about storing.');
             return;
         }
 
-        if (permission == Ephy.Permission.UNDECIDED && Ephy.isWebApplication())
+        if (permission === Ephy.Permission.UNDECIDED && Ephy.isWebApplication())
             permission = Ephy.Permission.PERMIT;
 
         Ephy.passwordManager.query(
@@ -553,12 +559,12 @@ Ephy.FormManager = class FormManager
             formAuth.usernameField,
             formAuth.passwordField).then(authInfo => {
                 if (authInfo) {
-                    if (authInfo.username == formAuth.username && authInfo.password == formAuth.password) {
+                    if (authInfo.username === formAuth.username && authInfo.password === formAuth.password) {
                         Ephy.log('User/password already stored. Not asking about storing.');
                         return;
                     }
 
-                    if (permission == Ephy.Permission.PERMIT) {
+                    if (permission === Ephy.Permission.PERMIT) {
                         Ephy.log('User/password not yet stored. Storing.');
                         Ephy.passwordManager.save(formAuth.origin,
                                                   formAuth.targetOrigin,
@@ -581,7 +587,7 @@ Ephy.FormManager = class FormManager
                                                  formAuth.password,
                                                  formAuth.usernameField,
                                                  formAuth.passwordField,
-                                                 authInfo == null,
+                                                 authInfo === null,
                                                  this._pageID);
             }
         );
@@ -592,7 +598,7 @@ Ephy.FormManager = class FormManager
     _containsPasswordElement()
     {
         for (let i = 0; i < this._form.elements.length; i++) {
-            let element = this._form.elements[i];
+            const element = this._form.elements[i];
             if (element instanceof HTMLInputElement) {
                 if (element.type === 'password' || element.type === 'adminpw')
                     return true;
@@ -605,31 +611,31 @@ Ephy.FormManager = class FormManager
     {
         let isFormActionInsecure = false;
         if (this._form.action) {
-            let url = new URL(this._form.action);
-            if (url.protocol == 'http:') {
+            const url = new URL(this._form.action);
+            if (url.protocol === 'http:') {
                 // We trust localhost to be local since glib!616.
-                let parts = url.hostname.split('.');
+                const parts = url.hostname.split('.');
                 if (parts.length > 0) {
-                    let tld = parts[parts.length - 1];
-                    isFormActionInsecure = tld != "127.0.0.1" && tld != "::1" && tld != "localhost";
+                    const tld = parts[parts.length - 1];
+                    isFormActionInsecure = tld !== '127.0.0.1' && tld !== '::1' && tld !== 'localhost';
                 }
             }
         }
-        window.webkit.messageHandlers.passwordFormFocused.postMessage(this._passwordFormMessageSerializer(this._frameID, isFormActionInsecure));
+        window.webkit.messageHandlers.passwordFormFocused.postMessage(this._passwordFormMessageSerializer(this._pageID, isFormActionInsecure));
     }
 
     _findPasswordFields()
     {
-        let passwordFields = [];
+        const passwordFields = [];
         for (let i = 0; i < this._form.elements.length; i++) {
-            let element = this._form.elements[i];
+            const element = this._form.elements[i];
             if (element instanceof HTMLInputElement && element.type === 'password') {
                 // We only want to process forms with 1-3 fields. A common
                 // case is to have a "change password" form with 3 fields:
                 // Old password, New password, Confirm new password.
                 // Forms with more than 3 password fields are unlikely,
                 // and we don't know how to process them, so reject them
-                if (passwordFields.length == 3)
+                if (passwordFields.length === 3)
                     return null;
                 passwordFields.push({ 'element' : element, 'index' : i });
             }
@@ -639,16 +645,16 @@ Ephy.FormManager = class FormManager
 
     _findFormAuthElements(forAutofill)
     {
-        let passwordNodes = this._findPasswordFields();
+        const passwordNodes = this._findPasswordFields();
         if (!passwordNodes || !passwordNodes.length)
             return null;
 
         // Start at the first found password field and search backwards.
         // Assume the first eligible field to contain username.
         let usernameNode = null;
-        let firstPasswordNodeData = passwordNodes[0];
+        const firstPasswordNodeData = passwordNodes[0];
         for (let i = firstPasswordNodeData.index; i >= 0; i--) {
-            let element = this._form.elements[i];
+            const element = this._form.elements[i];
             if (element instanceof HTMLInputElement) {
                 if (element.type === 'text' || element.type === 'email' ||
                     element.type === 'tel' || element.type === 'url' ||
@@ -664,41 +670,41 @@ Ephy.FormManager = class FormManager
         // submits login data, because otherwise all the fields are empty. In that
         // case just pick the first field.
         let passwordNodeIndex = 0;
-        if (!forAutofill && passwordNodes.length != 1) {
+        if (!forAutofill && passwordNodes.length !== 1) {
             // Get values of all password fields.
-            let passwords = [];
+            const passwords = [];
             for (let i = passwordNodes.length - 1; i >= 0; i--)
                 passwords[i] = passwordNodes[i].element.value;
 
-            if (passwordNodes.length == 2) {
+            if (passwordNodes.length === 2) {
                 // If there are two password fields, assume that the form has either
                 // Password and Confirm password fields, or Old password and New password.
                 // That can be guessed by comparing values in the fields. If they are
                 // different, we assume that the second password is "new" and use it.
                 // If they match, then just take the first field.
-                if (passwords[0] == passwords[1]) {
+                if (passwords[0] === passwords[1]) {
                     // Password / Confirm password.
                     passwordNodeIndex = 0;
                 } else {
                     // Old password / New password.
                     passwordNodeIndex = 1;
                 }
-            } else if (passwordNodes.length == 3) {
+            } else if (passwordNodes.length === 3) {
                 // This is probably a complete Old password, New password, Confirm
                 // new password case. Here we assume that if two fields have the same
                 // value, then it's the new password and we should take it. A special
                 // case is when all 3 passwords are different. We don't know what to
                 // do in this case, so just reject the form.
-                if (passwords[0] == passwords[1] && passwords[1] == passwords[2]) {
+                if (passwords[0] === passwords[1] && passwords[1] === passwords[2]) {
                     // All values are same.
                     passwordNodeIndex = 0;
-                } else if (passwords[0] == passwords[1]) {
+                } else if (passwords[0] === passwords[1]) {
                     // New password / Confirm new password / Old password.
                     passwordNodeIndex = 0;
-                } else if (passwords[0] == passwords[2]) {
+                } else if (passwords[0] === passwords[2]) {
                     // New password / Old password / Confirm new password.
                     passwordNodeIndex = 0;
-                } else if (passwords[1] == passwords[2]) {
+                } else if (passwords[1] === passwords[2]) {
                     // Old password / New password / Confirm new password.
                     passwordNodeIndex = 1;
                 } else {
@@ -719,8 +725,8 @@ Ephy.FormManager = class FormManager
     }
 
     _generateFormAuth(forAutofill) {
-        let formAuth = this._findFormAuthElements(forAutofill);
-        if (formAuth == null)
+        const formAuth = this._findFormAuthElements(forAutofill);
+        if (formAuth === null)
             return null;
 
         formAuth.origin = new URL(String(window.location)).origin;
@@ -757,4 +763,4 @@ Ephy.FormManager = class FormManager
 
         return formAuth;
     }
-}
+};
