@@ -29,36 +29,24 @@
 #include <errno.h>
 #include <gio/gio.h>
 #include <glib/gstdio.h>
-#include <libsoup/soup.h>
 #include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
 
-/* Web apps are installed in the default data dir of the user. Every
- * app has its own profile directory. To create a web app, an ID needs
- * to be generated using the given name as <normalized-name>-checksum.
- * The ID is used to uniquely identify the app.
+/* Web Apps are installed in the default config dir of the user.
+ * Every app has its own profile directory. To create a web app
+ * an id needs to be generated using the given name as
+ * <normalized-name>-checksum.
  *
- *  - Name: the user-visible pretty app name
- *  - Normalized name: lowercase name
- *  - Checksum: SHA-1 of name
- *  - ID: <normalized-name>-<checksum>
- *  - GApplication ID: see below
- *  - Profile directory: <gapplication-id>
- *  - Desktop file: <profile-dir>/<gapplication-id>.desktop
- *
- * Note that our ID and GApplication ID are different. Yes, this is confusing.
- *
- * For GApplication ID, there are two cases:
- *
- *  - If <id> is safe to use in a D-Bus identifier,
- *    then: GApplication ID is org.gnome.Epiphany.WebApp-<id>
- *  - otherwise: GAppliation ID is org.gnome.Epiphany.WebApp-<checksum>
+ * The id is used to uniquely identify the app.
+ *  - Program name: org.gnome.Epiphany.WebApp-<id>
+ *  - Profile directory: <program-name>
+ *  - Desktop file: <profile-dir>/<program-name>.desktop
  *
  * System web applications have a profile dir without a desktop file.
  */
 
-#define EPHY_WEB_APP_GAPPLICATION_ID_PREFIX "org.gnome.Epiphany.WebApp-"
+#define EPHY_WEB_APP_PROGRAM_NAME_PREFIX "org.gnome.Epiphany.WebApp-"
 
 char *
 ephy_web_application_get_app_id_from_name (const char *name)
@@ -96,97 +84,34 @@ get_encoded_path (const char *path)
   return encoded;
 }
 
-static const char *
-get_app_id_from_gapplication_id (const char *name)
-{
-  if (!g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX)) {
-    g_warning ("GApplication ID %s does not begin with required prefix %s", name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX);
-    return NULL;
-  }
-
-  return name + strlen (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX);
-}
-
-static char *
-get_gapplication_id_from_id (const char *id)
-{
-  g_autofree char *gapplication_id = NULL;
-  const char *final_hyphen;
-  const char *checksum;
-
-  /* Ideally we would convert hyphens to underscores here, because
-   * hyphens are not very friendly to D-Bus. However, changing this
-   * would require a new profile migration, because the GApplication ID
-   * must exactly match the desktop file basename.
-   *
-   * If we're willing to do another migration in the future, then we
-   * should drop this path and always return just the prefix plus hash,
-   * using underscores rather than hyphens.
-   */
-  gapplication_id = g_strconcat (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX, id, NULL);
-  if (g_application_id_is_valid (gapplication_id))
-    return g_steal_pointer (&gapplication_id);
-
-  /* Split ID into: <normalized-name>-<checksum> */
-  final_hyphen = strrchr (id, '-');
-  if (!final_hyphen) {
-    g_warning ("Web app ID %s is broken: must contain a hyphen", id);
-    return NULL;
-  }
-  checksum = final_hyphen + 1;
-
-  if (*checksum == '\0') {
-    g_warning ("Web app ID %s is broken: should end with checksum, not hyphen", id);
-    return NULL;
-  }
-
-  /* We'll simply omit the <normalized-name> from the app ID, in order
-   * to avoid problematic characters. Ideally we would use an underscore
-   * here too, rather than a hyphen, but let's be consistent with
-   * existing web apps.
-   */
-  g_clear_pointer (&gapplication_id, g_free);
-  gapplication_id = g_strconcat (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX, checksum, NULL);
-
-  if (!g_application_id_is_valid (gapplication_id)) {
-    g_warning ("Web app ID %s is broken: derived GApplication ID %s is not a valid app ID (is the final component alphanumeric?)", id, gapplication_id);
-    return NULL;
-  }
-
-  return g_steal_pointer (&gapplication_id);
-}
-
 static char *
 get_app_profile_directory_name (const char *id)
 {
-  g_autofree char *gapplication_id = NULL;
+  char *profile_dir;
+  char *encoded;
 
-  gapplication_id = get_gapplication_id_from_id (id);
-  if (!gapplication_id)
-    g_error ("Failed to get GApplication ID from app ID %s", id);
+  profile_dir = g_strconcat (EPHY_WEB_APP_PROGRAM_NAME_PREFIX, id, NULL);
+  encoded = get_encoded_path (profile_dir);
+  g_free (profile_dir);
 
-  return get_encoded_path (gapplication_id);
+  return encoded;
 }
 
 static char *
 get_app_desktop_filename (const char *id)
 {
-  g_autofree char *gapplication_id = NULL;
-  g_autofree char *filename = NULL;
+  char *filename;
+  char *encoded;
 
-  /* Warning: the GApplication ID must exactly match the desktop file's
-   * basename. Don't overthink this or stuff will break, e.g. GNotification.
-   */
-  gapplication_id = get_gapplication_id_from_id (id);
-  if (!gapplication_id)
-    g_error ("Failed to get GApplication ID from app ID %s", id);
+  filename = g_strconcat (EPHY_WEB_APP_PROGRAM_NAME_PREFIX, id, ".desktop", NULL);
+  encoded = get_encoded_path (filename);
+  g_free (filename);
 
-  filename = g_strconcat (gapplication_id, ".desktop", NULL);
-  return get_encoded_path (filename);
+  return encoded;
 }
 
 const char *
-ephy_web_application_get_gapplication_id_from_profile_directory (const char *profile_dir)
+ephy_web_application_get_program_name_from_profile_directory (const char *profile_dir)
 {
   const char *name;
 
@@ -203,21 +128,32 @@ ephy_web_application_get_gapplication_id_from_profile_directory (const char *pro
   if (g_str_has_prefix (name, "app-"))
     name += strlen ("app-");
 
-  if (!g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX)) {
-    g_warning ("Profile directory %s does not begin with required web app prefix %s", profile_dir, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX);
+  if (!g_str_has_prefix (name, EPHY_WEB_APP_PROGRAM_NAME_PREFIX)) {
+    g_warning ("Profile directory %s does not begin with required web app prefix %s", profile_dir, EPHY_WEB_APP_PROGRAM_NAME_PREFIX);
     return NULL;
   }
 
   return name;
 }
 
+static const char *
+get_app_id_from_program_name (const char *name)
+{
+  if (!g_str_has_prefix (name, EPHY_WEB_APP_PROGRAM_NAME_PREFIX)) {
+    g_warning ("Program name %s does not begin with required prefix %s", name, EPHY_WEB_APP_PROGRAM_NAME_PREFIX);
+    return NULL;
+  }
+
+  return name + strlen (EPHY_WEB_APP_PROGRAM_NAME_PREFIX);
+}
+
 static const char *
 get_app_id_from_profile_directory (const char *profile_dir)
 {
-  const char *gapplication_id;
+  const char *program_name;
 
-  gapplication_id = ephy_web_application_get_gapplication_id_from_profile_directory (profile_dir);
-  return gapplication_id ? get_app_id_from_gapplication_id (gapplication_id) : NULL;
+  program_name = ephy_web_application_get_program_name_from_profile_directory (profile_dir);
+  return program_name ? get_app_id_from_program_name (program_name) : NULL;
 }
 
 static char *
@@ -386,7 +322,7 @@ create_desktop_file (const char *id,
     g_free (path);
   }
 
-  wm_class = g_strconcat (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX, id, NULL);
+  wm_class = g_strconcat (EPHY_WEB_APP_PROGRAM_NAME_PREFIX, id, NULL);
   g_key_file_set_value (file, "Desktop Entry", "StartupWMClass", wm_class);
   g_free (wm_class);
 
@@ -542,7 +478,7 @@ ephy_web_application_ensure_for_app_info (GAppInfo *app_info)
 void
 ephy_web_application_setup_from_profile_directory (const char *profile_directory)
 {
-  const char *gapplication_id;
+  const char *program_name;
   const char *id;
   char *app_icon;
   char *desktop_basename;
@@ -551,23 +487,24 @@ ephy_web_application_setup_from_profile_directory (const char *profile_directory
 
   g_assert (profile_directory != NULL);
 
-  gapplication_id = ephy_web_application_get_gapplication_id_from_profile_directory (profile_directory);
-  if (!gapplication_id)
-    g_error ("Failed to get GApplication ID from profile directory %s", profile_directory);
+  program_name = ephy_web_application_get_program_name_from_profile_directory (profile_directory);
+  if (!program_name)
+    exit (1);
 
-  /* FIXME: is this actually necessary? Probably not? */
-  g_set_prgname (gapplication_id);
+  g_set_prgname (program_name);
 
-  id = get_app_id_from_gapplication_id (gapplication_id);
+  id = get_app_id_from_program_name (program_name);
   if (!id)
-    g_error ("Failed to get app ID from GApplication ID %s", gapplication_id);
+    exit (1);
 
   /* Get display name from desktop file */
   desktop_basename = get_app_desktop_filename (id);
   desktop_filename = g_build_filename (profile_directory, desktop_basename, NULL);
   desktop_info = g_desktop_app_info_new_from_filename (desktop_filename);
-  if (!desktop_info)
-    g_error ("Required desktop file not present at %s", desktop_filename);
+  if (!desktop_info) {
+    g_warning ("Required desktop file not present at %s", desktop_filename);
+    exit (1);
+  }
   g_set_application_name (g_app_info_get_name (G_APP_INFO (desktop_info)));
 
   app_icon = g_build_filename (profile_directory, EPHY_WEB_APP_ICON_NAME, NULL);
@@ -576,7 +513,7 @@ ephy_web_application_setup_from_profile_directory (const char *profile_directory
   /* We need to re-set this because we have already parsed the
    * options, which inits GTK+ and sets this as a side effect.
    */
-  gdk_set_program_class (gapplication_id);
+  gdk_set_program_class (program_name);
 
   g_free (app_icon);
   g_free (desktop_basename);
@@ -719,7 +656,7 @@ ephy_web_application_get_application_list_internal (gboolean only_legacy)
 
     name = g_file_info_get_name (info);
     if ((only_legacy && g_str_has_prefix (name, "app-")) ||
-        (!only_legacy && g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX))) {
+        (!only_legacy && g_str_has_prefix (name, EPHY_WEB_APP_PROGRAM_NAME_PREFIX))) {
       EphyWebApplication *app;
       char *profile_dir;
 
@@ -879,29 +816,31 @@ static gboolean
 urls_have_same_origin (const char *a_url,
                        const char *b_url)
 {
-  SoupURI *a_uri, *b_uri;
-  gboolean retval = FALSE;
+  g_autoptr (GUri) a_uri = NULL;
+  g_autoptr (GUri) b_uri = NULL;
 
-  a_uri = soup_uri_new (a_url);
-  if (!a_uri)
-    return retval;
+  a_uri = g_uri_parse (a_url, G_URI_FLAGS_NONE, NULL);
+  if (!a_uri || !g_uri_get_host (a_uri))
+    return FALSE;
 
-  b_uri = soup_uri_new (b_url);
-  if (b_uri) {
-    retval = a_uri->host && b_uri->host && soup_uri_host_equal (a_uri, b_uri);
-    soup_uri_free (b_uri);
-  }
+  b_uri = g_uri_parse (b_url, G_URI_FLAGS_NONE, NULL);
+  if (!b_uri || !g_uri_get_host (b_uri))
+    return FALSE;
 
-  soup_uri_free (a_uri);
+  if (strcmp (g_uri_get_scheme (a_uri), g_uri_get_scheme (b_uri)) != 0)
+    return FALSE;
 
-  return retval;
+  if (g_uri_get_port (a_uri) != g_uri_get_port (b_uri))
+    return FALSE;
+
+  return g_ascii_strcasecmp (g_uri_get_host (a_uri), g_uri_get_host (b_uri)) == 0;
 }
 
 gboolean
 ephy_web_application_is_uri_allowed (const char *uri)
 {
   EphyWebApplication *webapp = ephy_web_application_for_profile_directory (ephy_profile_dir ());
-  SoupURI *request_uri;
+  const char *scheme;
   char **urls;
   guint i;
   gboolean matched = FALSE;
@@ -917,25 +856,24 @@ ephy_web_application_is_uri_allowed (const char *uri)
   if (g_strcmp0 (uri, "about:blank") == 0)
     return TRUE;
 
-  request_uri = soup_uri_new (uri);
-  if (!request_uri)
+  scheme = g_uri_peek_scheme (uri);
+  if (!scheme)
     return FALSE;
 
   urls = g_settings_get_strv (EPHY_SETTINGS_WEB_APP, EPHY_PREFS_WEB_APP_ADDITIONAL_URLS);
   for (i = 0; urls[i] && !matched; i++) {
     if (!strstr (urls[i], "://")) {
-      char *url = g_strdup_printf ("%s://%s", request_uri->scheme, urls[i]);
+      g_autofree char *url = NULL;
+
+      url = g_strdup_printf ("%s://%s", scheme, urls[i]);
 
       matched = g_str_has_prefix (uri, url);
-      g_free (url);
     } else {
       matched = g_str_has_prefix (uri, urls[i]);
     }
   }
   g_strfreev (urls);
 
-  soup_uri_free (request_uri);
-
   return matched;
 }
 
