@@ -21,13 +21,13 @@
 #include "config.h"
 #include "ephy-sync-crypto.h"
 
+#include "ephy-debug.h"
 #include "ephy-string.h"
 #include "ephy-sync-utils.h"
 
 #include <glib/gstdio.h>
 #include <inttypes.h>
 #include <json-glib/json-glib.h>
-#include <libsoup/soup.h>
 #include <nettle/aes.h>
 #include <nettle/cbc.h>
 #include <nettle/hkdf.h>
@@ -282,7 +282,7 @@ ephy_sync_crypto_hawk_header_new (const char            *url,
 {
   SyncCryptoHawkHeader *hawk_header;
   SyncCryptoHawkArtifacts *artifacts;
-  SoupURI *uri;
+  g_autoptr (GUri) uri = NULL;
   char *resource;
   char *hash;
   char *header;
@@ -302,12 +302,12 @@ ephy_sync_crypto_hawk_header_new (const char            *url,
   hash = options ? g_strdup (options->hash) : NULL;
   payload = options ? options->payload : NULL;
   timestamp = options ? options->timestamp : NULL;
-  uri = soup_uri_new (url);
-  resource = !soup_uri_get_query (uri) ? g_strdup (soup_uri_get_path (uri))
-                                       : g_strconcat (soup_uri_get_path (uri),
-                                                      "?",
-                                                      soup_uri_get_query (uri),
-                                                      NULL);
+  uri = g_uri_parse (url, G_URI_FLAGS_SCHEME_NORMALIZE, NULL);
+  resource = !g_uri_get_query (uri) ? g_strdup (g_uri_get_path (uri))
+                                    : g_strconcat (g_uri_get_path (uri),
+                                                   "?",
+                                                   g_uri_get_query (uri),
+                                                   NULL);
 
   if (options && options->nonce) {
     nonce = g_strdup (options->nonce);
@@ -339,10 +339,10 @@ ephy_sync_crypto_hawk_header_new (const char            *url,
                                                    options ? options->dlg : NULL,
                                                    options ? options->ext : NULL,
                                                    hash,
-                                                   soup_uri_get_host (uri),
+                                                   g_uri_get_host (uri),
                                                    method,
                                                    nonce,
-                                                   soup_uri_get_port (uri),
+                                                   g_uri_get_port (uri),
                                                    resource,
                                                    ts);
 
@@ -385,7 +385,6 @@ ephy_sync_crypto_hawk_header_new (const char            *url,
   hawk_header->header = g_strdup (header);
   hawk_header->artifacts = artifacts;
 
-  soup_uri_free (uri);
   g_free (hash);
   g_free (mac);
   g_free (nonce);
@@ -693,15 +692,15 @@ ephy_sync_crypto_derive_master_keys (const char    *bundle_hex,
                                      guint8       **ka,
                                      guint8       **kb)
 {
+  gsize len = 32; /* The master sync keys are always 32 bytes. */
   guint8 *bundle;
-  guint8 *ciphertext;
-  guint8 *resp_hmac;
+  guint8 ciphertext[2 * len];
+  guint8 resp_hmac[len];
   guint8 *resp_hmac_2;
   guint8 *xored;
   guint8 *wrap_kb;
   char *resp_hmac_2_hex;
   gboolean retval = TRUE;
-  gsize len = 32; /* The master sync keys are always 32 bytes. */
 
   g_assert (bundle_hex);
   g_assert (resp_hmac_key);
@@ -711,8 +710,6 @@ ephy_sync_crypto_derive_master_keys (const char    *bundle_hex,
   g_assert (kb);
 
   bundle = ephy_sync_utils_decode_hex (bundle_hex);
-  ciphertext = g_malloc (2 * len);
-  resp_hmac = g_malloc (len);
 
   /* Compute the MAC and compare it to the expected value. */
   memcpy (ciphertext, bundle, 2 * len);
@@ -743,8 +740,6 @@ ephy_sync_crypto_derive_master_keys (const char    *bundle_hex,
 out:
   g_free (resp_hmac_2);
   g_free (resp_hmac_2_hex);
-  g_free (resp_hmac);
-  g_free (ciphertext);
   g_free (bundle);
 
   return retval;
@@ -1112,19 +1107,19 @@ ephy_sync_crypto_decrypt_record (const char          *payload,
   /* Extract ciphertext, iv and hmac from payload. */
   node = json_from_string (payload, &error);
   if (error) {
-    g_warning ("Payload is not a valid JSON: %s", error->message);
+    LOG ("Payload is not a valid JSON: %s", error->message);
     goto out;
   }
   json = json_node_get_object (node);
   if (!json) {
-    g_warning ("JSON node does not hold a JSON object");
+    LOG ("JSON node does not hold a JSON object");
     goto out;
   }
   ciphertext_b64 = json_object_get_string_member (json, "ciphertext");
   iv_b64 = json_object_get_string_member (json, "IV");
   hmac = json_object_get_string_member (json, "hmac");
   if (!ciphertext_b64 || !iv_b64 || !hmac) {
-    g_warning ("JSON object has missing or invalid members");
+    LOG ("JSON object has missing or invalid members");
     goto out;
   }
 
@@ -1136,7 +1131,7 @@ ephy_sync_crypto_decrypt_record (const char          *payload,
    * if the HMAC verification fails.
    */
   if (!ephy_sync_crypto_hmac_is_valid (ciphertext_b64, hmac_key, hmac)) {
-    g_warning ("Incorrect HMAC value");
+    LOG ("Incorrect HMAC value");
     goto out;
   }
 
