@@ -880,6 +880,7 @@ ephy_shell_dispose (GObject *object)
   g_clear_object (&shell->bookmarks_manager);
   g_clear_object (&shell->history_manager);
   g_clear_object (&shell->open_tabs_manager);
+  g_clear_object (&shell->web_extension_manager);
 
   g_hash_table_destroy (shell->notifications);
 
@@ -1289,10 +1290,9 @@ _ephy_shell_create_instance (EphyEmbedShellMode mode)
 
   if (mode == EPHY_EMBED_SHELL_MODE_APPLICATION) {
     const char *profile_dir = ephy_profile_dir ();
+    const char *web_id = ephy_web_application_get_program_name_from_profile_directory (profile_dir);
 
-    id = ephy_web_application_get_gapplication_id_from_profile_directory (profile_dir);
-    if (id == NULL)
-      g_error ("Cannot start web app instance: %s is not a valid profile directory", profile_dir);
+    id = web_id;
   } else {
     id = APPLICATION_ID;
   }
