@@ -43,6 +43,13 @@ struct _EphySuggestionModel {
   SoupSession *session;
 };
 
+#define QUERY_SCOPE_ALL         ' '
+#define QUERY_SCOPE_TABS        '%'
+#define QUERY_SCOPE_SUGGESTIONS '?'
+#define QUERY_SCOPE_BOOKMARKS   '*'
+#define QUERY_SCOPE_HISTORY     '^'
+#define MAX_QUERY_SCOPES        4
+
 enum {
   PROP_0,
   PROP_BOOKMARKS_MANAGER,
@@ -293,8 +300,12 @@ append_suggestion (EphySuggestionModel *self,
     return FALSE;
 
   if (self->num_custom_entries < MAX_URL_ENTRIES) {
+    const char *uri = ephy_suggestion_get_uri (suggestion);
+
     g_sequence_append (self->items, g_object_ref (suggestion));
-    load_favicon (self, suggestion, ephy_suggestion_get_uri (suggestion));
+    g_sequence_append (self->urls, g_strdup (uri));
+
+    load_favicon (self, suggestion, uri);
     self->num_custom_entries++;
 
     return TRUE;
@@ -303,61 +314,6 @@ append_suggestion (EphySuggestionModel *self,
   return FALSE;
 }
 
-static guint
-add_bookmarks (EphySuggestionModel *self,
-               const char          *query)
-{
-  g_autoptr (GList) new_urls = NULL;
-  GSequence *bookmarks;
-  guint added = 0;
-
-  bookmarks = ephy_bookmarks_manager_get_bookmarks (self->bookmarks_manager);
-
-  for (GSequenceIter *iter = g_sequence_get_begin_iter (bookmarks);
-       !g_sequence_iter_is_end (iter);
-       iter = g_sequence_iter_next (iter)) {
-    EphyBookmark *bookmark;
-    GSequence *tags;
-    const char *url, *title;
-
-    bookmark = g_sequence_get (iter);
-
-    url = ephy_bookmark_get_url (bookmark);
-    if (g_sequence_lookup (self->urls, (gpointer)url, (GCompareDataFunc)g_strcmp0, NULL))
-      continue;
-
-    title = ephy_bookmark_get_title (bookmark);
-    if (strlen (title) == 0)
-      title = url;
-
-    tags = ephy_bookmark_get_tags (bookmark);
-    if (should_add_bookmark_to_model (self, query, title, url, tags)) {
-      EphySuggestion *suggestion;
-      g_autofree gchar *escaped_title = NULL;
-      g_autofree gchar *markup = NULL;
-
-      escaped_title = g_markup_escape_text (title, -1);
-      markup = dzl_fuzzy_highlight (escaped_title, query, FALSE);
-      suggestion = ephy_suggestion_new (markup, title, url);
-      load_favicon (self, suggestion, url);
-      ephy_suggestion_set_secondary_icon (suggestion, "starred-symbolic");
-
-      if (append_suggestion (self, suggestion)) {
-        new_urls = g_list_prepend (new_urls, g_strdup (url));
-        added++;
-      } else {
-        break;
-      }
-    }
-  }
-
-  for (GList *p = new_urls; p != NULL; p = p->next)
-    g_sequence_append (self->urls, g_steal_pointer (&p->data));
-  g_sequence_sort (self->urls, (GCompareDataFunc)g_strcmp0, NULL);
-
-  return added;
-}
-
 static guint
 add_search_engines (EphySuggestionModel *self,
                     const char          *query)
@@ -376,17 +332,17 @@ add_search_engines (EphySuggestionModel *self,
     g_autofree char *address = NULL;
     g_autofree char *escaped_title = NULL;
     g_autofree char *markup = NULL;
-    g_autoptr (SoupURI) uri = NULL;
+    g_autoptr (GUri) uri = NULL;
 
     address = ephy_search_engine_manager_build_search_address (manager, engines[i], query);
     escaped_title = g_markup_escape_text (engines[i], -1);
     markup = dzl_fuzzy_highlight (escaped_title, query, FALSE);
     suggestion = ephy_suggestion_new (markup, engines[i], address);
 
-    uri = soup_uri_new (address);
+    uri = g_uri_parse (address, G_URI_FLAGS_NONE, NULL);
     if (uri) {
       g_free (address);
-      address = g_strdup_printf ("%s://%s/", soup_uri_get_scheme (uri), soup_uri_get_host (uri));
+      address = g_strdup_printf ("%s://%s/", g_uri_get_scheme (uri), g_uri_get_host (uri));
     }
 
     load_favicon (self, suggestion, address);
@@ -399,10 +355,143 @@ add_search_engines (EphySuggestionModel *self,
 
   return added;
 }
+typedef struct {
+  char *query;
+  char scope;
+  gboolean include_search_engines;
+  GSequence *tabs;
+  GSequence *bookmarks;
+  GSequence *history;
+  GSequence *google_suggestions;
+  int active_sources;
+} QueryData;
 
-static guint
-add_tabs (EphySuggestionModel *self,
-          const char          *query)
+static QueryData *
+query_data_new (const char *query,
+                gboolean    include_search_engines)
+{
+  QueryData *data;
+
+  data = g_malloc0 (sizeof (QueryData));
+  data->include_search_engines = include_search_engines;
+  data->tabs = g_sequence_new (g_object_unref);
+  data->bookmarks = g_sequence_new (g_object_unref);
+  data->history = g_sequence_new (g_object_unref);
+  data->google_suggestions = g_sequence_new (g_object_unref);
+
+  /* Split the search string. */
+  if (strlen (query) > 1 && query[1] == ' ' &&
+      (query[0] == QUERY_SCOPE_TABS ||
+       query[0] == QUERY_SCOPE_BOOKMARKS ||
+       query[0] == QUERY_SCOPE_HISTORY ||
+       query[0] == QUERY_SCOPE_SUGGESTIONS)) {
+    data->query = g_strdup (query + 2);
+    data->scope = query[0];
+    data->active_sources = 1;
+  } else {
+    data->query = g_strdup (query);
+    data->scope = QUERY_SCOPE_ALL;
+    data->active_sources = MAX_QUERY_SCOPES;
+  }
+
+  return data;
+}
+
+static void
+query_data_free (QueryData *data)
+{
+  g_assert (data != NULL);
+  g_clear_pointer (&data->tabs, g_sequence_free);
+  g_clear_pointer (&data->bookmarks, g_sequence_free);
+  g_clear_pointer (&data->history, g_sequence_free);
+  g_clear_pointer (&data->google_suggestions, g_sequence_free);
+  g_clear_pointer (&data->query, g_free);
+  g_free (data);
+}
+
+static void
+query_collection_done (EphySuggestionModel *self,
+                       GTask               *task)
+{
+  QueryData *data;
+  guint removed;
+  guint added = 0;
+
+  self = g_task_get_source_object (task);
+  data = g_task_get_task_data (task);
+
+  if (--data->active_sources)
+    return;
+
+  g_cancellable_cancel (self->icon_cancellable);
+  g_clear_object (&self->icon_cancellable);
+
+  self->icon_cancellable = g_cancellable_new ();
+
+  removed = g_sequence_get_length (self->items);
+
+  g_clear_pointer (&self->urls, g_sequence_free);
+  self->urls = g_sequence_new (g_free);
+  g_clear_pointer (&self->items, g_sequence_free);
+  self->items = g_sequence_new (g_object_unref);
+  self->num_custom_entries = 0;
+
+  if (strlen (data->query) > 0) {
+    /* Search results have the following order:
+     * - Open Tabs
+     * - Search Suggestions
+     * - Bookmarks
+     * - History
+     * - Search Engines
+     */
+    for (GSequenceIter *iter = g_sequence_get_begin_iter (data->tabs); !g_sequence_iter_is_end (iter); iter = g_sequence_iter_next (iter)) {
+      EphySuggestion *tmp = g_sequence_get (iter);
+
+      g_sequence_append (self->items, g_object_ref (tmp));
+      added++;
+    }
+
+    for (GSequenceIter *iter = g_sequence_get_begin_iter (data->google_suggestions); !g_sequence_iter_is_end (iter); iter = g_sequence_iter_next (iter)) {
+      EphySuggestion *tmp = g_sequence_get (iter);
+
+      if (append_suggestion (self, tmp))
+        added++;
+      else
+        break;
+    }
+
+    for (GSequenceIter *iter = g_sequence_get_begin_iter (data->bookmarks); !g_sequence_iter_is_end (iter); iter = g_sequence_iter_next (iter)) {
+      EphySuggestion *tmp = g_sequence_get (iter);
+
+      if (append_suggestion (self, tmp))
+        added++;
+      else
+        break;
+    }
+
+    for (GSequenceIter *iter = g_sequence_get_begin_iter (data->history); !g_sequence_iter_is_end (iter); iter = g_sequence_iter_next (iter)) {
+      EphySuggestion *tmp = g_sequence_get (iter);
+
+      if (append_suggestion (self, tmp))
+        added++;
+      else
+        break;
+    }
+
+    if (data->scope == QUERY_SCOPE_ALL && data->include_search_engines)
+      added += add_search_engines (self, data->query);
+  }
+
+  g_list_model_items_changed (G_LIST_MODEL (self), 0, removed, added);
+
+  g_task_return_boolean (task, TRUE);
+  g_object_unref (task);
+}
+
+static void
+tabs_query (EphySuggestionModel *self,
+            QueryData           *data,
+            GTask               *task)
 {
   GApplication *application;
   EphyEmbedShell *shell;
@@ -410,7 +499,6 @@ add_tabs (EphySuggestionModel *self,
   EphyTabView *tab_view;
   GList *windows;
   gint n_pages, selected;
-  guint added = 0;
 
   shell = ephy_embed_shell_get_default ();
   application = G_APPLICATION (shell);
@@ -430,7 +518,6 @@ add_tabs (EphySuggestionModel *self,
       g_autofree gchar *escaped_title = NULL;
       g_autofree gchar *markup = NULL;
       const gchar *display_address;
-      const gchar *url;
       g_autofree gchar *address = NULL;
       const gchar *title;
       g_autofree gchar *title_casefold = NULL;
@@ -443,12 +530,11 @@ add_tabs (EphySuggestionModel *self,
       embed = EPHY_EMBED (ephy_tab_view_get_nth_page (tab_view, i));
       webview = ephy_embed_get_web_view (embed);
       display_address = ephy_web_view_get_display_address (webview);
-      url = ephy_web_view_get_address (webview);
       address = g_strdup_printf ("ephy-tab://%d@%d", i, win_idx);
       title = webkit_web_view_get_title (WEBKIT_WEB_VIEW (webview));
 
       display_address_casefold = g_utf8_casefold (display_address, -1);
-      query_casefold = g_utf8_casefold (query, -1);
+      query_casefold = g_utf8_casefold (data->query, -1);
       if (!title)
         title = "";
 
@@ -458,116 +544,58 @@ add_tabs (EphySuggestionModel *self,
         char *escaped_address = g_markup_escape_text (display_address, -1);
 
         escaped_title = g_markup_escape_text (title, -1);
-        markup = dzl_fuzzy_highlight (escaped_title, query, FALSE);
+        markup = dzl_fuzzy_highlight (escaped_title, data->query, FALSE);
         suggestion = ephy_suggestion_new_with_custom_subtitle (markup, title, escaped_address, address);
-        load_favicon (self, suggestion, display_address);
         ephy_suggestion_set_secondary_icon (suggestion, "go-jump-symbolic");
 
-        g_sequence_append (self->urls, g_strdup (url));
-        g_sequence_append (self->items, suggestion);
-        added++;
+        g_sequence_append (data->tabs, g_object_ref (suggestion));
       }
     }
   }
 
-  g_sequence_sort (self->urls, (GCompareDataFunc)g_strcmp0, NULL);
-
-  return added;
-}
-
-typedef struct {
-  char *query;
-  gboolean include_search_engines;
-  GSequence *history;
-  GSequence *google_suggestions;
-  int active_sources;
-} QueryData;
-
-static QueryData *
-query_data_new (const char *query,
-                gboolean    include_search_engines)
-{
-  QueryData *data;
-
-  data = g_malloc0 (sizeof (QueryData));
-  data->query = g_strdup (query);
-  data->include_search_engines = include_search_engines;
-  data->history = g_sequence_new (g_object_unref);
-  data->google_suggestions = g_sequence_new (g_object_unref);
-
-  return data;
+  query_collection_done (self, g_steal_pointer (&task));
 }
 
 static void
-query_data_free (QueryData *data)
+bookmarks_query (EphySuggestionModel *self,
+                 QueryData           *data,
+                 GTask               *task)
 {
-  g_assert (data != NULL);
-  g_clear_pointer (&data->history, g_sequence_free);
-  g_clear_pointer (&data->google_suggestions, g_sequence_free);
-  g_free (data->query);
-  g_free (data);
-}
+  g_autoptr (GList) new_urls = NULL;
+  GSequence *bookmarks;
 
-static void
-query_collection_done (EphySuggestionModel *self,
-                       GTask               *task)
-{
-  QueryData *data;
-  guint removed;
-  guint added = 0;
+  bookmarks = ephy_bookmarks_manager_get_bookmarks (self->bookmarks_manager);
 
-  self = g_task_get_source_object (task);
-  data = g_task_get_task_data (task);
+  for (GSequenceIter *iter = g_sequence_get_begin_iter (bookmarks);
+       !g_sequence_iter_is_end (iter);
+       iter = g_sequence_iter_next (iter)) {
+    EphyBookmark *bookmark;
+    GSequence *tags;
+    const char *url, *title;
 
-  if (--data->active_sources) {
-    g_object_unref (task);
-    return;
+    bookmark = g_sequence_get (iter);
+
+    url = ephy_bookmark_get_url (bookmark);
+    title = ephy_bookmark_get_title (bookmark);
+    if (strlen (title) == 0)
+      title = url;
+
+    tags = ephy_bookmark_get_tags (bookmark);
+    if (should_add_bookmark_to_model (self, data->query, title, url, tags)) {
+      EphySuggestion *suggestion;
+      g_autofree gchar *escaped_title = NULL;
+      g_autofree gchar *markup = NULL;
+
+      escaped_title = g_markup_escape_text (title, -1);
+      markup = dzl_fuzzy_highlight (escaped_title, data->query, FALSE);
+      suggestion = ephy_suggestion_new (markup, title, url);
+      ephy_suggestion_set_secondary_icon (suggestion, "starred-symbolic");
+
+      g_sequence_append (data->bookmarks, g_object_ref (suggestion));
+    }
   }
 
-  g_cancellable_cancel (self->icon_cancellable);
-  g_clear_object (&self->icon_cancellable);
-
-  self->icon_cancellable = g_cancellable_new ();
-
-  removed = g_sequence_get_length (self->items);
-
-  g_clear_pointer (&self->urls, g_sequence_free);
-  self->urls = g_sequence_new (g_free);
-  g_clear_pointer (&self->items, g_sequence_free);
-  self->items = g_sequence_new (g_object_unref);
-  self->num_custom_entries = 0;
-
-  if (strlen (data->query) > 0) {
-    added = add_tabs (self, data->query);
-
-    for (GSequenceIter *iter = g_sequence_get_begin_iter (data->google_suggestions); !g_sequence_iter_is_end (iter); iter = g_sequence_iter_next (iter)) {
-      EphySuggestion *tmp = g_sequence_get (iter);
-
-      if (append_suggestion (self, tmp))
-        added++;
-      else
-        break;
-    }
-
-    added += add_bookmarks (self, data->query);
-
-    for (GSequenceIter *iter = g_sequence_get_begin_iter (data->history); !g_sequence_iter_is_end (iter); iter = g_sequence_iter_next (iter)) {
-      EphySuggestion *tmp = g_sequence_get (iter);
-
-      if (append_suggestion (self, tmp))
-        added++;
-      else
-        break;
-    }
-
-    if (data->include_search_engines)
-      added += add_search_engines (self, data->query);
-  }
-
-  g_list_model_items_changed (G_LIST_MODEL (self), 0, removed, added);
-
-  g_task_return_boolean (task, TRUE);
-  g_object_unref (task);
+  query_collection_done (self, g_steal_pointer (&task));
 }
 
 static void
@@ -608,10 +636,17 @@ history_query_completed_cb (EphyHistoryService *service,
   query_collection_done (self, g_steal_pointer (&task));
 }
 
+#if SOUP_CHECK_VERSION (2, 99, 4)
+static void
+google_search_suggestions_cb (SoupSession  *session,
+                              GAsyncResult *result,
+                              gpointer      user_data)
+#else
 static void
 google_search_suggestions_cb (SoupSession *session,
                               SoupMessage *msg,
                               gpointer     user_data)
+#endif
 {
   GTask *task = G_TASK (user_data);
   EphySuggestionModel *self = g_task_get_source_object (task);
@@ -623,17 +658,33 @@ google_search_suggestions_cb (SoupSession *session,
   JsonArray *suggestions;
   char *engine;
   int added = 0;
+  g_autoptr (GBytes) body = NULL;
+#if SOUP_CHECK_VERSION (2, 99, 4)
+  SoupMessage *msg;
 
+  body = soup_session_send_and_read_finish (session, result, NULL);
+  if (!body)
+    goto out;
+
+  msg = soup_session_get_async_result_message (session, result);
+  if (soup_message_get_status (msg) != 200)
+    goto out;
+
+  /* FIXME: we don't have a way to get the status code */
+#else
   if (msg->status_code != 200)
     goto out;
 
+  body = g_bytes_new_static (msg->response_body->data, msg->response_body->length);
+#endif
+
   shell = ephy_embed_shell_get_default ();
   manager = ephy_embed_shell_get_search_engine_manager (shell);
   engine = ephy_search_engine_manager_get_default_engine (manager);
 
-  node = json_from_string (msg->response_body->data, NULL);
+  node = json_from_string (g_bytes_get_data (body, NULL), NULL);
   if (!node || !JSON_NODE_HOLDS_ARRAY (node)) {
-    g_warning ("Google search suggestion response is not a valid JSON object: %s", msg->response_body->data);
+    g_warning ("Google search suggestion response is not a valid JSON object: %s", (const char *)g_bytes_get_data (body, NULL));
     goto out;
   }
 
@@ -679,8 +730,14 @@ google_search_suggestions_query (EphySuggestionModel *self,
   escaped_query = g_markup_escape_text (query, -1);
   url = g_strdup_printf ("http://suggestqueries.google.com/complete/search?client=firefox&q=%s", escaped_query);
   msg = soup_message_new (SOUP_METHOD_GET, url);
-
+#if SOUP_CHECK_VERSION (2, 99, 4)
+  soup_session_send_and_read_async (self->session, msg, G_PRIORITY_DEFAULT, NULL,
+                                    (GAsyncReadyCallback)google_search_suggestions_cb,
+                                    g_steal_pointer (&task));
+  g_object_unref (msg);
+#else
   soup_session_queue_message (self->session, g_steal_pointer (&msg), google_search_suggestions_cb, g_steal_pointer (&task));
+#endif
 }
 
 void
@@ -692,8 +749,6 @@ ephy_suggestion_model_query_async (EphySuggestionModel *self,
                                    gpointer             user_data)
 {
   GTask *task = NULL;
-  char **strings;
-  GList *qlist = NULL;
   QueryData *data;
 
   g_assert (EPHY_IS_SUGGESTION_MODEL (self));
@@ -706,30 +761,38 @@ ephy_suggestion_model_query_async (EphySuggestionModel *self,
   data = query_data_new (query, include_search_engines);
   g_task_set_task_data (task, data, (GDestroyNotify)query_data_free);
 
-  /* Split the search string. */
-  strings = g_strsplit (query, " ", -1);
-  for (guint i = 0; strings[i]; i++)
-    qlist = g_list_append (qlist, g_strdup (strings[i]));
-
-  /* History */
-  data->active_sources = 1;
-
   /* Google Search Suggestions */
-  if (g_settings_get_boolean (EPHY_SETTINGS_MAIN, EPHY_PREFS_USE_GOOGLE_SEARCH_SUGGESTIONS)) {
-    data->active_sources++;
-    google_search_suggestions_query (self, query, g_object_ref (task));
+  if (data->scope == QUERY_SCOPE_ALL || data->scope == QUERY_SCOPE_SUGGESTIONS) {
+    if (g_settings_get_boolean (EPHY_SETTINGS_MAIN, EPHY_PREFS_USE_GOOGLE_SEARCH_SUGGESTIONS))
+      google_search_suggestions_query (self, query, task);
+    else
+      query_collection_done (self, task);
   }
 
-  ephy_history_service_find_urls (self->history_service,
-                                  0, 0,
-                                  MAX_URL_ENTRIES, 0,
-                                  qlist,
-                                  EPHY_HISTORY_SORT_MOST_VISITED,
-                                  cancellable,
-                                  (EphyHistoryJobCallback)history_query_completed_cb,
-                                  task);
+  if (data->scope == QUERY_SCOPE_ALL || data->scope == QUERY_SCOPE_HISTORY) {
+    GList *qlist = NULL;
+    g_auto (GStrv) strings = NULL;
 
-  g_strfreev (strings);
+    strings = g_strsplit (data->query, " ", -1);
+
+    for (guint i = 0; strings[i]; i++)
+      qlist = g_list_append (qlist, g_strdup (strings[i]));
+
+    ephy_history_service_find_urls (self->history_service,
+                                    0, 0,
+                                    MAX_URL_ENTRIES, 0,
+                                    qlist,
+                                    EPHY_HISTORY_SORT_MOST_VISITED,
+                                    cancellable,
+                                    (EphyHistoryJobCallback)history_query_completed_cb,
+                                    task);
+  }
+
+  if (data->scope == QUERY_SCOPE_ALL || data->scope == QUERY_SCOPE_TABS)
+    tabs_query (self, data, task);
+
+  if (data->scope == QUERY_SCOPE_ALL || data->scope == QUERY_SCOPE_BOOKMARKS)
+    bookmarks_query (self, data, task);
 }
 
 gboolean
