@@ -46,22 +46,22 @@
 #include "ephy-history-dialog.h"
 #include "ephy-link.h"
 #include "ephy-location-entry.h"
-#include "ephy-notebook.h"
 #include "ephy-password-import.h"
 #include "ephy-prefs.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-string.h"
+#include "ephy-tab-view.h"
 #include "ephy-view-source-handler.h"
 #include "ephy-web-app-utils.h"
+#include "ephy-web-extension-dialog.h"
 #include "ephy-zoom.h"
 
 #include <gio/gio.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libsoup/soup.h>
 #include <string.h>
 #include <webkit2/webkit2.h>
 
@@ -137,22 +137,24 @@ static gchar *
 get_path (GIOChannel *channel)
 {
   gchar *line;
-  gchar *path;
+  const gchar *path;
+  gchar *result;
   gsize length;
 
   do {
     g_io_channel_read_line (channel, &line, &length, NULL, NULL);
 
     if (g_str_has_prefix (line, "Path")) {
-      path = g_strdup (line);
+      path = line;
 
       /* Extract value (e.g. Path=Value\n -> Value) */
       path = strchr (path, '=');
       path++;
-      path[strcspn (path, "\n")] = 0;
+      ((gchar *)path)[strcspn (path, "\n")] = '\0';
+      result = g_strdup (path);
 
       g_free (line);
-      return path;
+      return result;
     }
 
     g_free (line);
@@ -646,10 +648,11 @@ window_cmd_export_bookmarks (GSimpleAction *action,
                                                           _("_Cancel")));
   gtk_file_chooser_set_show_hidden (dialog, TRUE);
 
-  /* Translators: Only translate the part before ".gvdb" (e.g. "bookmarks") */
-  gtk_file_chooser_set_current_name (dialog, _("bookmarks.gvdb"));
+  /* Translators: Only translate the part before ".html" (e.g. "bookmarks") */
+  gtk_file_chooser_set_current_name (dialog, _("bookmarks.html"));
 
   filter = gtk_file_filter_new ();
+  gtk_file_filter_add_pattern (filter, "*.html");
   gtk_file_filter_add_pattern (filter, "*.gvdb");
   gtk_file_chooser_set_filter (dialog, filter);
 
@@ -870,6 +873,21 @@ window_cmd_show_history (GSimpleAction *action,
   gtk_window_present_with_time (GTK_WINDOW (dialog), gtk_get_current_event_time ());
 }
 
+void
+window_cmd_show_firefox_sync (GSimpleAction *action,
+                              GVariant      *parameter,
+                              gpointer       user_data)
+{
+  GtkWidget *dialog;
+
+  dialog = ephy_shell_get_firefox_sync_dialog (ephy_shell_get_default ());
+
+  if (GTK_WINDOW (user_data) != gtk_window_get_transient_for (GTK_WINDOW (dialog)))
+    gtk_window_set_transient_for (GTK_WINDOW (dialog),
+                                  GTK_WINDOW (user_data));
+  gtk_window_present_with_time (GTK_WINDOW (dialog), gtk_get_current_event_time ());
+}
+
 void
 window_cmd_show_preferences (GSimpleAction *action,
                              GVariant      *parameter,
@@ -900,7 +918,7 @@ window_cmd_show_shortcuts (GSimpleAction *action,
     shortcuts_window = GTK_WIDGET (gtk_builder_get_object (builder, "shortcuts-dialog"));
 
     if (ephy_is_running_inside_flatpak ())
-      gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "shortcuts-web-apps-group")));
+      gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (builder, "shortcuts-web-apps-group")));
 
     if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL) {
       GtkShortcutsShortcut *shortcut;
@@ -978,7 +996,7 @@ window_cmd_show_about (GSimpleAction *action,
   g_key_file_free (key_file);
 
   comments = g_strdup_printf (_("A simple, clean, beautiful view of the web.\n"
-                                "Powered by WebKitGTK %d.%d.%d"),
+                                "Powered by WebKitGTK %d.%d.%d" WEBKIT_REVISION),
                               webkit_get_major_version (),
                               webkit_get_minor_version (),
                               webkit_get_micro_version ());
@@ -988,16 +1006,20 @@ window_cmd_show_about (GSimpleAction *action,
   if (window)
     gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
 
+  if (g_str_equal (PROFILE, "Canary"))
+    gtk_about_dialog_set_program_name (dialog, _("Epiphany Canary"));
+  else {
 #if !TECH_PREVIEW
-  gtk_about_dialog_set_program_name (dialog, _("Web"));
+    gtk_about_dialog_set_program_name (dialog, _("Web"));
 #else
-  gtk_about_dialog_set_program_name (dialog, _("Epiphany Technology Preview"));
+    gtk_about_dialog_set_program_name (dialog, _("Epiphany Technology Preview"));
 #endif
+  }
 
   gtk_about_dialog_set_version (dialog, VERSION);
   gtk_about_dialog_set_copyright (dialog,
                                   "Copyright © 2002–2004 Marco Pesenti Gritti\n"
-                                  "Copyright © 2003–2020 The GNOME Web Developers");
+                                  "Copyright © 2003–2021 The GNOME Web Developers");
   gtk_about_dialog_set_comments (dialog, comments);
   gtk_about_dialog_set_license_type (dialog, GTK_LICENSE_GPL_3_0);
   gtk_about_dialog_set_website (dialog, "https://wiki.gnome.org/Apps/Web");
@@ -1160,6 +1182,49 @@ window_cmd_stop (GSimpleAction *action,
   webkit_web_view_stop_loading (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 }
 
+static void
+check_tab_has_modified_forms_and_reload_cb (EphyWebView  *view,
+                                            GAsyncResult *result,
+                                            EphyEmbed    *embed)
+{
+  EphyWindow *window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view)));
+  GtkWidget *dialog;
+  GtkWidget *button;
+  gboolean has_modified_forms;
+  int response = GTK_RESPONSE_ACCEPT;
+
+  has_modified_forms = ephy_web_view_has_modified_forms_finish (view, result, NULL);
+  if (has_modified_forms) {
+    dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                                     GTK_DIALOG_MODAL,
+                                     GTK_MESSAGE_WARNING,
+                                     GTK_BUTTONS_CANCEL,
+                                     "%s", _("Do you want to reload this website?"));
+
+    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", _("A form you modified has not been submitted."));
+
+    button = gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Discard form"), GTK_RESPONSE_ACCEPT);
+    gtk_style_context_add_class (gtk_widget_get_style_context (button), "destructive-action");
+
+    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
+
+    gtk_window_group_add_window (gtk_window_get_group (GTK_WINDOW (window)),
+                                 GTK_WINDOW (dialog));
+    response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+    gtk_widget_destroy (dialog);
+  }
+
+  if (response == GTK_RESPONSE_ACCEPT) {
+    WebKitWebView *view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+
+    gtk_widget_grab_focus (GTK_WIDGET (embed));
+    webkit_web_view_reload (view);
+  }
+
+  g_object_unref (embed);
+}
+
 void
 window_cmd_reload (GSimpleAction *action,
                    GVariant      *parameter,
@@ -1167,15 +1232,14 @@ window_cmd_reload (GSimpleAction *action,
 {
   EphyWindow *window = EPHY_WINDOW (user_data);
   EphyEmbed *embed;
-  WebKitWebView *view;
 
-  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  embed = EPHY_EMBED (ephy_tab_view_get_current_page (ephy_window_get_tab_view (window)));
   g_assert (embed != NULL);
 
-  gtk_widget_grab_focus (GTK_WIDGET (embed));
-
-  view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
-  webkit_web_view_reload (view);
+  ephy_web_view_has_modified_forms (ephy_embed_get_web_view (embed),
+                                    NULL,
+                                    (GAsyncReadyCallback)check_tab_has_modified_forms_and_reload_cb,
+                                    g_object_ref (embed));
 }
 
 void
@@ -1196,9 +1260,10 @@ window_cmd_reload_bypass_cache (GSimpleAction *action,
   webkit_web_view_reload_bypass_cache (view);
 }
 
-void window_cmd_combined_stop_reload (GSimpleAction *action,
-                                      GVariant      *parameter,
-                                      gpointer       user_data)
+void
+window_cmd_combined_stop_reload (GSimpleAction *action,
+                                 GVariant      *parameter,
+                                 gpointer       user_data)
 {
   GActionGroup *action_group;
   GAction *gaction;
@@ -1308,7 +1373,7 @@ typedef struct {
   char *icon_href;
   GdkRGBA icon_rgba;
   GCancellable *cancellable;
-  gboolean mobile_capable;
+  EphyWebApplicationOptions webapp_options;
   WebKitDownload *download;
 } EphyApplicationDialogData;
 
@@ -1583,11 +1648,11 @@ set_default_application_title (EphyApplicationDialogData *data,
                                char                      *title)
 {
   if (title == NULL || title[0] == '\0') {
-    SoupURI *uri;
+    g_autoptr (GUri) uri = NULL;
     const char *host;
 
-    uri = soup_uri_new (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)));
-    host = soup_uri_get_host (uri);
+    uri = g_uri_parse (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)), G_URI_FLAGS_NONE, NULL);
+    host = g_uri_get_host (uri);
 
     if (host != NULL && host[0] != '\0') {
       title = get_special_case_application_title_for_host (host);
@@ -1599,8 +1664,6 @@ set_default_application_title (EphyApplicationDialogData *data,
           title = g_strdup (host);
       }
     }
-
-    soup_uri_free (uri);
   }
 
   if (title == NULL || title[0] == '\0') {
@@ -1643,7 +1706,7 @@ fill_mobile_capable_cb (GObject      *source,
 
   capable = ephy_web_view_get_web_app_mobile_capable_finish (EPHY_WEB_VIEW (source), async_result, &error);
   if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-    data->mobile_capable = capable;
+    data->webapp_options = capable ? EPHY_WEB_APPLICATION_MOBILE_CAPABLE : EPHY_WEB_APPLICATION_NONE;
 }
 
 static void
@@ -1761,7 +1824,7 @@ dialog_save_as_application_response_cb (GtkDialog                 *dialog,
                                                 webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)),
                                                 app_name,
                                                 gtk_image_get_pixbuf (GTK_IMAGE (data->image)),
-                                                data->mobile_capable);
+                                                data->webapp_options);
 
     if (desktop_file)
       message = g_strdup_printf (_("The application “%s” is ready to be used"),
@@ -1897,36 +1960,35 @@ static char *
 get_suggested_filename (EphyEmbed *embed)
 {
   EphyWebView *view;
-  char *suggested_filename = NULL;
+  const char *suggested_filename;
   const char *mimetype;
   WebKitURIResponse *response;
   WebKitWebResource *web_resource;
-  SoupURI *soup_uri;
+  g_autoptr (GUri) uri = NULL;
 
   view = ephy_embed_get_web_view (embed);
   web_resource = webkit_web_view_get_main_resource (WEBKIT_WEB_VIEW (view));
   response = webkit_web_resource_get_response (web_resource);
   mimetype = webkit_uri_response_get_mime_type (response);
-  soup_uri = soup_uri_new (webkit_web_resource_get_uri (web_resource));
+  uri = g_uri_parse (webkit_web_resource_get_uri (web_resource), G_URI_FLAGS_SCHEME_NORMALIZE, NULL);
 
-  if (g_ascii_strncasecmp (mimetype, "text/html", 9) == 0 && g_strcmp0 (soup_uri_get_scheme (soup_uri), EPHY_VIEW_SOURCE_SCHEME) != 0) {
+  if (g_ascii_strncasecmp (mimetype, "text/html", 9) == 0 && g_strcmp0 (g_uri_get_scheme (uri), EPHY_VIEW_SOURCE_SCHEME) != 0) {
     /* Web Title will be used as suggested filename */
-    suggested_filename = g_strconcat (ephy_embed_get_title (embed), ".mhtml", NULL);
-  } else {
-    suggested_filename = g_strdup (webkit_uri_response_get_suggested_filename (response));
-    if (!suggested_filename) {
-      char *last_slash = strrchr (soup_uri->path, '/');
-      suggested_filename = soup_uri_decode (last_slash ? (last_slash + 1) : soup_uri->path);
-
-      if (!strlen (suggested_filename)) {
-        g_free (suggested_filename);
-        suggested_filename = g_strdup ("index.html");
-      }
-    }
+    return g_strconcat (ephy_embed_get_title (embed), ".mhtml", NULL);
   }
 
-  soup_uri_free (soup_uri);
-  return suggested_filename;
+  suggested_filename = webkit_uri_response_get_suggested_filename (response);
+  if (!suggested_filename) {
+    const char *path = g_uri_get_path (uri);
+    char *last_slash = strrchr (path, '/');
+    if (last_slash)
+      path = last_slash + 1;
+
+    if (path[0] != '\0')
+      return g_strdup (path);
+  }
+
+  return suggested_filename ? g_strdup (suggested_filename) : g_strdup ("index.html");
 }
 
 
@@ -2311,6 +2373,34 @@ window_cmd_bookmark_page (GSimpleAction *action,
   ephy_add_bookmark_popover_show (EPHY_ADD_BOOKMARK_POPOVER (popover));
 }
 
+void
+window_cmd_bookmarks (GSimpleAction *action,
+                      GVariant      *parameter,
+                      gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyHeaderBar *header_bar;
+  EphyActionBarEnd *action_bar_end;
+
+  header_bar = EPHY_HEADER_BAR (ephy_window_get_header_bar (window));
+  action_bar_end = EPHY_ACTION_BAR_END (ephy_header_bar_get_action_bar_end (header_bar));
+  ephy_action_bar_end_show_bookmarks (action_bar_end);
+}
+
+void
+window_cmd_show_downloads (GSimpleAction *action,
+                           GVariant      *parameter,
+                           gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyHeaderBar *header_bar;
+  EphyActionBarEnd *action_bar_end;
+
+  header_bar = EPHY_HEADER_BAR (ephy_window_get_header_bar (window));
+  action_bar_end = EPHY_ACTION_BAR_END (ephy_header_bar_get_action_bar_end (header_bar));
+  ephy_action_bar_end_show_downloads (action_bar_end);
+}
+
 void
 window_cmd_zoom_in (GSimpleAction *action,
                     GVariant      *parameter,
@@ -2361,10 +2451,10 @@ window_cmd_page_source (GSimpleAction *action,
   EphyWindow *window = user_data;
   EphyEmbed *embed;
   EphyEmbed *new_embed;
-  SoupURI *soup_uri;
+  g_autoptr (GUri) uri = NULL;
+  g_autoptr (GUri) converted_uri = NULL;
   char *source_uri;
   const char *address;
-  guint port;
 
   embed = ephy_embed_container_get_active_child
             (EPHY_EMBED_CONTAINER (window));
@@ -2376,20 +2466,22 @@ window_cmd_page_source (GSimpleAction *action,
   if (strstr (address, EPHY_VIEW_SOURCE_SCHEME) == address)
     return;
 
-  soup_uri = soup_uri_new (address);
-  if (!soup_uri) {
-    g_critical ("Failed to construct SoupURI for %s", address);
+  uri = g_uri_parse (address, G_URI_FLAGS_ENCODED | G_URI_FLAGS_SCHEME_NORMALIZE, NULL);
+  if (!uri) {
+    g_critical ("Failed to construct GUri for %s", address);
     return;
   }
 
-  /* Convert e.g. https://gnome.org to ephy-source://gnome.org#https,
-   * taking care to prevent soup_uri_set_scheme() from forcing the default port.
-   */
-  port = soup_uri_get_port (soup_uri);
-  soup_uri_set_fragment (soup_uri, soup_uri->scheme);
-  soup_uri_set_scheme (soup_uri, EPHY_VIEW_SOURCE_SCHEME);
-  soup_uri_set_port (soup_uri, port);
-  source_uri = soup_uri_to_string (soup_uri, FALSE);
+  /* Convert e.g. https://gnome.org to ephy-source://gnome.org#https */
+  converted_uri = g_uri_build (g_uri_get_flags (uri),
+                               EPHY_VIEW_SOURCE_SCHEME,
+                               g_uri_get_userinfo (uri),
+                               g_uri_get_host (uri),
+                               g_uri_get_port (uri),
+                               g_uri_get_path (uri),
+                               g_uri_get_query (uri),
+                               g_uri_get_scheme (uri));
+  source_uri = g_uri_to_string (converted_uri);
 
   new_embed = ephy_shell_new_tab
                 (ephy_shell_get_default (),
@@ -2401,7 +2493,6 @@ window_cmd_page_source (GSimpleAction *action,
   gtk_widget_grab_focus (GTK_WIDGET (new_embed));
 
   g_free (source_uri);
-  soup_uri_free (soup_uri);
 }
 
 void
@@ -2595,80 +2686,21 @@ window_cmd_change_fullscreen_state (GSimpleAction *action,
   g_simple_action_set_state (action, g_variant_new_boolean (active));
 }
 
-void
-window_cmd_tabs_previous (GSimpleAction *action,
-                          GVariant      *variant,
-                          gpointer       user_data)
-{
-  GtkWidget *nb;
-
-  nb = ephy_window_get_notebook (EPHY_WINDOW (user_data));
-  g_assert (nb != NULL);
-
-  ephy_notebook_prev_page (EPHY_NOTEBOOK (nb));
-}
-
-void
-window_cmd_tabs_next (GSimpleAction *action,
-                      GVariant      *variant,
-                      gpointer       user_data)
-{
-  GtkWidget *nb;
-
-  nb = ephy_window_get_notebook (EPHY_WINDOW (user_data));
-  g_assert (nb != NULL);
-
-  ephy_notebook_next_page (EPHY_NOTEBOOK (nb));
-}
-
-void
-window_cmd_tabs_move_left (GSimpleAction *action,
-                           GVariant      *variant,
-                           gpointer       user_data)
-{
-  GtkWidget *child;
-  GtkNotebook *notebook;
-  int page;
-
-  notebook = GTK_NOTEBOOK (ephy_window_get_notebook (EPHY_WINDOW (user_data)));
-  page = gtk_notebook_get_current_page (notebook);
-  if (page < 1)
-    return;
-
-  child = gtk_notebook_get_nth_page (notebook, page);
-  gtk_notebook_reorder_child (notebook, child, page - 1);
-}
-
-void window_cmd_tabs_move_right (GSimpleAction *action,
-                                 GVariant      *variant,
-                                 gpointer       user_data)
-{
-  GtkWidget *child;
-  GtkNotebook *notebook;
-  int page, n_pages;
-
-  notebook = GTK_NOTEBOOK (ephy_window_get_notebook (EPHY_WINDOW (user_data)));
-  page = gtk_notebook_get_current_page (notebook);
-  n_pages = gtk_notebook_get_n_pages (notebook) - 1;
-  if (page > n_pages - 1)
-    return;
-
-  child = gtk_notebook_get_nth_page (notebook, page);
-  gtk_notebook_reorder_child (notebook, child, page + 1);
-}
-
 void
 window_cmd_tabs_duplicate (GSimpleAction *action,
                            GVariant      *variant,
                            gpointer       user_data)
 {
+  EphyTabView *tab_view;
   EphyEmbed *embed, *new_embed;
   WebKitWebView *view, *new_view;
   WebKitWebViewSessionState *session_state;
   WebKitBackForwardList *bf_list;
   WebKitBackForwardListItem *item;
 
-  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (user_data));
+  tab_view = ephy_window_get_tab_view (EPHY_WINDOW (user_data));
+  embed = EPHY_EMBED (ephy_tab_view_get_current_page (tab_view));
+
   view = WEBKIT_WEB_VIEW (ephy_embed_get_web_view (embed));
   session_state = webkit_web_view_get_session_state (view);
 
@@ -2690,56 +2722,23 @@ window_cmd_tabs_duplicate (GSimpleAction *action,
     ephy_web_view_load_url (EPHY_WEB_VIEW (new_view), webkit_web_view_get_uri (view));
 }
 
-void
-window_cmd_tabs_detach (GSimpleAction *action,
-                        GVariant      *variant,
-                        gpointer       user_data)
-{
-  EphyEmbed *embed;
-  GtkNotebook *notebook;
-  EphyWindow *new_window;
-
-  notebook = GTK_NOTEBOOK (ephy_window_get_notebook (EPHY_WINDOW (user_data)));
-  if (gtk_notebook_get_n_pages (notebook) <= 1)
-    return;
-
-  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (user_data));
-
-  g_object_ref_sink (embed);
-  gtk_notebook_remove_page (notebook, gtk_notebook_page_num (notebook, GTK_WIDGET (embed)));
-
-  new_window = ephy_window_new ();
-  ephy_embed_container_add_child (EPHY_EMBED_CONTAINER (new_window), embed, 0, FALSE);
-  g_object_unref (embed);
-
-  gtk_window_present_with_time (GTK_WINDOW (new_window), gtk_get_current_event_time ());
-}
-
 void
 window_cmd_tabs_close (GSimpleAction *action,
                        GVariant      *parameter,
                        gpointer       user_data)
 {
   EphyWindow *window = user_data;
-  GtkWidget *notebook;
-  EphyEmbed *embed;
-  gboolean pinned;
+  EphyTabView *tab_view;
 
-  notebook = ephy_window_get_notebook (window);
+  tab_view = ephy_window_get_tab_view (window);
 
   if (g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
                               EPHY_PREFS_LOCKDOWN_QUIT) &&
-      gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook)) == 1) {
+      ephy_tab_view_get_n_pages (tab_view) <= 1) {
     return;
   }
 
-  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-  g_assert (embed != NULL);
-
-  pinned = ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), embed);
-
-  if (!pinned)
-    g_signal_emit_by_name (notebook, "tab-close-request", embed);
+  ephy_tab_view_close_selected (tab_view);
 }
 
 void
@@ -2748,27 +2747,8 @@ window_cmd_tabs_close_left (GSimpleAction *action,
                             gpointer       user_data)
 {
   EphyWindow *window = user_data;
-  GtkWidget *notebook;
-  EphyEmbed *embed;
-  int current_page_no;
-  GSList *pages_to_close = NULL;
 
-  notebook = ephy_window_get_notebook (window);
-  current_page_no = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
-
-  for (int i = 0; i < current_page_no; i++) {
-    embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
-
-    if (!ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), embed))
-      pages_to_close = g_slist_prepend (pages_to_close, embed);
-  }
-
-  for (GSList *l = pages_to_close; l != NULL; l = l->next) {
-    g_assert (l->data != NULL);
-    g_signal_emit_by_name (GTK_NOTEBOOK (notebook), "tab-close-request", l->data);
-  }
-
-  g_slist_free (pages_to_close);
+  ephy_tab_view_close_left (ephy_window_get_tab_view (window));
 }
 
 void
@@ -2777,28 +2757,8 @@ window_cmd_tabs_close_right (GSimpleAction *action,
                              gpointer       user_data)
 {
   EphyWindow *window = user_data;
-  GtkWidget *notebook;
-  EphyEmbed *embed;
-  int n_pages, current_page_no;
-  GSList *pages_to_close = NULL;
 
-  notebook = ephy_window_get_notebook (window);
-  n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
-  current_page_no = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
-
-  for (int i = current_page_no + 1; i < n_pages; i++) {
-    embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
-
-    if (!ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), embed))
-      pages_to_close = g_slist_prepend (pages_to_close, embed);
-  }
-
-  for (GSList *l = pages_to_close; l != NULL; l = l->next) {
-    g_assert (l->data != NULL);
-    g_signal_emit_by_name (GTK_NOTEBOOK (notebook), "tab-close-request", l->data);
-  }
-
-  g_slist_free (pages_to_close);
+  ephy_tab_view_close_right (ephy_window_get_tab_view (window));
 }
 
 void
@@ -2807,47 +2767,17 @@ window_cmd_tabs_close_others (GSimpleAction *action,
                               gpointer       user_data)
 {
   EphyWindow *window = user_data;
-  GtkWidget *notebook;
-  EphyEmbed *embed;
-  int n_pages, current_page;
-  GSList *pages_to_close = NULL;
 
-  notebook = ephy_window_get_notebook (window);
-  n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
-  current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
-
-  for (int i = 0; i < n_pages; i++) {
-    if (i != current_page) {
-      embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
-
-      if (!ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), embed))
-        pages_to_close = g_slist_prepend (pages_to_close, embed);
-    }
-  }
-
-  for (GSList *l = pages_to_close; l != NULL; l = l->next) {
-    g_assert (l->data != NULL);
-    g_signal_emit_by_name (GTK_NOTEBOOK (notebook), "tab-close-request", l->data);
-  }
-
-  g_slist_free (pages_to_close);
+  ephy_tab_view_close_other (ephy_window_get_tab_view (window));
 }
 
-void
-window_cmd_tabs_reload (GSimpleAction *action,
-                        GVariant      *parameter,
-                        gpointer       user_data)
+static void
+reload_cb (GtkWidget *widget,
+           gpointer   user_data)
 {
-  EphyWindow *window = user_data;
-  EphyEmbed *embed;
-  WebKitWebView *view;
+  EphyEmbed *embed = EPHY_EMBED (widget);
+  WebKitWebView *view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
 
-  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-  g_assert (embed != NULL);
-
-  gtk_widget_grab_focus (GTK_WIDGET (embed));
-
-  view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
   webkit_web_view_reload (view);
 }
 
@@ -2857,56 +2787,10 @@ window_cmd_tabs_reload_all_tabs (GSimpleAction *action,
                                  gpointer       user_data)
 {
   EphyWindow *window = user_data;
-  EphyEmbed *embed;
-  WebKitWebView *view;
-  GtkWidget *notebook;
-  int n_pages;
 
-  notebook = ephy_window_get_notebook (window);
-  n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
-
-  for (int i = 0; i < n_pages; i++) {
-    embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
-
-    view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
-    webkit_web_view_reload (view);
-  }
-}
-
-void
-window_cmd_tabs_reopen_closed_tab (GSimpleAction *action,
-                                   GVariant      *parameter,
-                                   gpointer       user_data)
-{
-  EphySession *session = ephy_shell_get_session (ephy_shell_get_default ());
-
-  g_assert (session != NULL);
-  ephy_session_undo_close_tab (session);
-}
-
-void
-window_cmd_show_tab (GSimpleAction *action,
-                     GVariant      *parameter,
-                     gpointer       user_data)
-{
-  EphyWindow *window = EPHY_WINDOW (user_data);
-  GtkWidget *notebook;
-  guint32 tab_num;
-
-  g_assert (g_variant_is_of_type (parameter, G_VARIANT_TYPE_UINT32));
-  tab_num = g_variant_get_uint32 (parameter);
-
-  notebook = ephy_window_get_notebook (window);
-  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), tab_num);
-  g_simple_action_set_state (action, parameter);
-}
-
-void
-window_cmd_change_show_tab_state (GSimpleAction *action,
-                                  GVariant      *parameter,
-                                  gpointer       user_data)
-{
-  /* This page intentionally left blank. */
+  ephy_tab_view_foreach (ephy_window_get_tab_view (window),
+                         reload_cb,
+                         NULL);
 }
 
 void
@@ -2941,8 +2825,10 @@ window_cmd_open_application_manager (GSimpleAction *action,
   EphyEmbed *embed;
   EphyWebView *web_view;
 
-  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-  g_assert (embed != NULL);
+  embed = ephy_shell_new_tab (ephy_shell_get_default (),
+                              window,
+                              NULL,
+                              EPHY_NEW_TAB_JUMP);
 
   web_view = ephy_embed_get_web_view (embed);
 
@@ -3019,15 +2905,8 @@ window_cmd_tabs_pin (GSimpleAction *action,
                      gpointer       user_data)
 {
   EphyWindow *window = EPHY_WINDOW (user_data);
-  EphyEmbed *embed;
-  EphyNotebook *notebook;
 
-  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-  g_assert (embed != NULL);
-
-  notebook = EPHY_NOTEBOOK (ephy_window_get_notebook (window));
-
-  ephy_notebook_tab_set_pinned (notebook, GTK_WIDGET (embed), TRUE);
+  ephy_tab_view_pin (ephy_window_get_tab_view (window));
 }
 
 void
@@ -3036,13 +2915,44 @@ window_cmd_tabs_unpin (GSimpleAction *action,
                        gpointer       user_data)
 {
   EphyWindow *window = EPHY_WINDOW (user_data);
-  EphyEmbed *embed;
-  EphyNotebook *notebook;
 
-  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  ephy_tab_view_unpin (ephy_window_get_tab_view (window));
+}
+
+void
+window_cmd_change_tabs_mute_state (GSimpleAction *action,
+                                   GVariant      *state,
+                                   gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyEmbed *embed;
+  EphyWebView *view;
+  gboolean mute;
+
+  embed = EPHY_EMBED (ephy_tab_view_get_current_page (ephy_window_get_tab_view (window)));
   g_assert (embed != NULL);
 
-  notebook = EPHY_NOTEBOOK (ephy_window_get_notebook (window));
+  view = ephy_embed_get_web_view (embed);
 
-  ephy_notebook_tab_set_pinned (notebook, GTK_WIDGET (embed), FALSE);
+  if (!webkit_web_view_is_playing_audio (WEBKIT_WEB_VIEW (view)))
+    return;
+
+  mute = !webkit_web_view_get_is_muted (WEBKIT_WEB_VIEW (view));
+
+  webkit_web_view_set_is_muted (WEBKIT_WEB_VIEW (view), mute);
+
+  g_simple_action_set_state (action, g_variant_new_boolean (mute));
+}
+
+void
+window_cmd_extensions (GSimpleAction *action,
+                       GVariant      *parameter,
+                       gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  GtkWidget *dialog;
+
+  dialog = ephy_web_extension_dialog_new ();
+  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
+  gtk_widget_show_all (dialog);
 }
