@@ -38,6 +38,14 @@
 #define HTML_STRING "testing-ephy-web-view"
 #define SERVER_PORT 12321
 
+#if SOUP_CHECK_VERSION (2, 99, 4)
+static void
+server_callback (SoupServer        *server,
+                 SoupServerMessage *msg,
+                 const char        *path,
+                 GHashTable        *query,
+                 gpointer           data)
+#else
 static void
 server_callback (SoupServer        *server,
                  SoupMessage       *msg,
@@ -45,20 +53,39 @@ server_callback (SoupServer        *server,
                  GHashTable        *query,
                  SoupClientContext *context,
                  gpointer           data)
+#endif
 {
-  if (!strcmp (path, "/cancelled"))
-    soup_message_set_status (msg, SOUP_STATUS_CANT_CONNECT);
-  else if (!strcmp (path, "/redirect")) {
+  SoupMessageHeaders *response_headers;
+  SoupMessageBody *response_body;
+
+#if SOUP_CHECK_VERSION (2, 99, 4)
+  response_headers = soup_server_message_get_response_headers (msg);
+  response_body = soup_server_message_get_response_body (msg);
+#else
+  response_headers = msg->response_headers;
+  response_body = msg->response_body;
+#endif
+
+  if (!strcmp (path, "/redirect")) {
+#if SOUP_CHECK_VERSION (2, 99, 4)
+    soup_server_message_set_status (msg, SOUP_STATUS_MOVED_PERMANENTLY, NULL);
+#else
     soup_message_set_status (msg, SOUP_STATUS_MOVED_PERMANENTLY);
-    soup_message_headers_append (msg->response_headers, "Location", "/redirect-result");
-  } else
+#endif
+    soup_message_headers_append (response_headers, "Location", "/redirect-result");
+  } else {
+#if SOUP_CHECK_VERSION (2, 99, 4)
+    soup_server_message_set_status (msg, SOUP_STATUS_OK, NULL);
+#else
     soup_message_set_status (msg, SOUP_STATUS_OK);
+#endif
+  }
 
 
-  soup_message_body_append (msg->response_body, SOUP_MEMORY_STATIC,
+  soup_message_body_append (response_body, SOUP_MEMORY_STATIC,
                             HTML_STRING, strlen (HTML_STRING));
 
-  soup_message_body_complete (msg->response_body);
+  soup_message_body_complete (response_body);
 }
 
 static void
@@ -129,7 +156,6 @@ static const URLTest test_load_url[] = {
   /* { "about:epiphany", "ephy-about:epiphany" }, */
   /* { "about:applications", "ephy-about:applications" }, */
   /* { "about:memory", "ephy-about:memory" }, */
-  /* { "about:plugins", "ephy-about:plugins" }, */
 };
 
 /* Tests that EphyWebView is successfully loading the given URL. */
@@ -142,8 +168,10 @@ test_ephy_web_view_load_url (void)
     URLTest test;
     GMainLoop *loop;
     EphyWebView *view;
+    GtkWidget *window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
     view = EPHY_WEB_VIEW (ephy_web_view_new ());
+    gtk_container_add (GTK_CONTAINER (window), GTK_WIDGET (view));
     test = test_load_url[i];
     loop = g_main_loop_new (NULL, FALSE);
 
@@ -317,11 +345,11 @@ test_ephy_web_view_normalize_or_autosearch (void)
                                             "http://www.google.com/?q=%s",
                                             "");
 
-  g_assert_true (ephy_search_engine_manager_set_default_engine (manager, default_engine));
   verify_normalize_or_autosearch_urls (view, normalize_or_autosearch_test_google, G_N_ELEMENTS (normalize_or_autosearch_test_google));
 
   ephy_search_engine_manager_delete_engine (manager, "org.gnome.Epiphany.EphyWebViewTest");
 
+  g_assert_true (ephy_search_engine_manager_set_default_engine (manager, default_engine));
   g_free (default_engine);
   g_object_unref (g_object_ref_sink (view));
 }
@@ -393,7 +421,7 @@ test_ephy_web_view_provisional_load_failure_updates_back_forward_list (void)
   view = EPHY_WEB_VIEW (ephy_web_view_new ());
 
   loop = setup_ensure_back_forward_list_changes (view);
-  bad_url = "http://localhost:2984375932/";
+  bad_url = "http://localhost:2984375930/";
 
   ephy_web_view_load_url (view, bad_url);
 
@@ -413,6 +441,9 @@ visit_url_cb (EphyHistoryService *service,
               EphyHistoryURL     *url,
               gpointer            user_data)
 {
+  if (strcmp (url->url, "http://localhost:2984375932/") != 0)
+    return;
+
   /* We are only loading an error page, this code should never be
    * reached. */
   g_assert_not_reached ();
